-- #############################################
-- ## VRIDRV Lua Universal Driver
-- ## Aerosoft Airbus X module Ver. 1n
-- #############################################


-- Default mode is MODE 1. 
-- If you dont need MODE 2, it's simple - just don't use it.


-- MCP buttons for MODE 1
MCP1 = {
["IDENT"]		= "Flgt"  ,		-- 4 chars to be shown on dsp, when enter this mode

["TOGA"] 		= Autobrake_cycle  ,
["TOGN"] 		= Antiskid_toggle  ,
["N1"] 			= empty  ,
["SPD"] 		= empty  ,
["FLCH"] 		= SPD_MACH_toggle  ,
["HDGSEL"] 		= empty  ,
["HDGHLD"] 		= empty  ,
["ALTHLD"] 		= empty  ,
["V/S FPA"] 	= empty  ,
["APP"] 		= Autopilot_APPR_toggle  ,
["VNAV"] 		= empty  ,
["LNAV"] 		= empty  ,
["CMDA"] 		= Autopilot_AP1_toggle  ,
["CMDB"] 		= Autopilot_AP2_toggle  ,
["CMDC"] 		= Autopilot_ATHR_toggle  ,
["LOC"] 		= Autopilot_LOC_toggle  ,
["CWSA"] 		= Autopilot_EXPED_toggle  ,
["CWSB"] 		= DSP_MODE_toggle  ,
["A/T UP"] 		= empty  ,
["A/T DN"] 		= empty  ,
["F/D UP"] 		= empty  ,
["F/D DN"] 		= empty  ,
["MASTER UP"] 	= empty  ,
["MASTER DN"] 	= empty  ,
}

-- MCP buttons for MODE 2
MCP2 = {
["IDENT"]		= "Grnd"  ,		-- 4 chars to be shown on dsp, when enter this mode

["TOGA"] 		= empty  ,
["TOGN"] 		= empty  ,
["N1"] 			= empty  ,
["SPD"] 		= empty  ,
["FLCH"] 		= empty  ,
["HDGSEL"] 		= empty  ,
["HDGHLD"] 		= empty  ,
["ALTHLD"] 		= empty  ,
["V/S FPA"] 	= empty  ,
["APP"] 		= empty  ,
["VNAV"] 		= empty  ,
["LNAV"] 		= empty  ,
["CMDA"] 		= empty  ,
["CMDB"] 		= empty  ,
["CMDC"] 		= empty  ,
["LOC"] 		= empty  ,
["CWSA"] 		= empty  ,
["CWSB"] 		= empty  ,
["A/T UP"] 		= empty  ,
["A/T DN"] 		= empty  ,
["F/D UP"] 		= empty  ,
["F/D DN"] 		= empty  ,
["MASTER UP"] 	= empty  ,
["MASTER DN"] 	= empty  ,
}


-- MCP buttons for MODE 3
MCP3 = {
["ENABLED"]		= false  , 		-- true / false to enable/disable MODE 3 for MCP controls
["IDENT"]		= "Othr"  ,		-- 4 chars to be shown on dsp, when enter this mode

["TOGA"] 		= empty  ,
["TOGN"] 		= empty  ,
["N1"] 			= empty  ,
["SPD"] 		= empty  ,
["FLCH"] 		= empty  ,
["HDGSEL"] 		= empty  ,
["HDGHLD"] 		= empty  ,
["ALTHLD"] 		= empty  ,
["V/S FPA"] 	= empty  ,
["APP"] 		= empty  ,
["VNAV"] 		= empty  ,
["LNAV"] 		= empty  ,
["CMDA"] 		= empty  ,
["CMDB"] 		= empty  ,
["CMDC"] 		= empty  ,
["LOC"] 		= empty  ,
["CWSA"] 		= empty  ,
["CWSB"] 		= empty  ,
["A/T UP"] 		= empty  ,
["A/T DN"] 		= empty  ,
["F/D UP"] 		= empty  ,
["F/D DN"] 		= empty  ,
["MASTER UP"] 	= empty  ,
["MASTER DN"] 	= empty  ,
}

-- ######################################################################### --



-- EFIS buttons for MODE 1
EFIS1 = {
["IDENT"]		= "Main"  ,		-- 4 chars to be shown on dsp, when enter this mode

["FPV"] 		= Alarms_WARN_off  ,
["MTR"] 		= Alarms_CAUTION_off  ,
["WX"] 			= Autopilot_FD_toggle  ,
["STA"] 		= Autopilot_LS_toggle  ,
["WPT"] 		= ND_CSTR_on  ,
["ARPT"] 		= ND_WPT_on  ,
["DATA"] 		= ND_VOR_on  ,
["POS"] 		= ND_NDB_on  ,
["TERR"] 		= ND_ARPT_on  ,
["ADF1"] 		= ND_ARROW1_adf  ,
["ADF2"] 		= ND_ARROW2_adf  ,
["VOR1"] 		= ND_ARROW1_vor  ,
["VOR2"] 		= ND_ARROW2_vor  ,
}

-- EFIS buttons for MODE 2
EFIS2 = {
["IDENT"]		= "WXR"  ,		-- 4 chars to be shown on dsp, when enter this mode

["FPV"] 		= empty  ,
["MTR"] 		= empty  ,
["WX"] 			= empty  ,
["STA"] 		= empty  ,
["WPT"] 		= empty  ,
["ARPT"] 		= empty  ,
["DATA"] 		= empty  ,
["POS"] 		= empty  ,
["TERR"] 		= empty  ,
["ADF1"] 		= empty  ,
["ADF2"] 		= empty  ,
["VOR1"] 		= empty  ,
["VOR2"] 		= empty  ,
}

-- EFIS buttons for MODE 3
EFIS3 = {
["ENABLED"]		= false  ,  	-- true / false to enable/disable MODE 3 for EFIS controls
["IDENT"]		= "Othr"  ,		-- 4 chars to be shown on dsp, when enter this mode

["FPV"] 		= empty  ,
["MTR"] 		= empty  ,
["WX"] 			= empty  ,
["STA"] 		= empty  ,
["WPT"] 		= empty  ,
["ARPT"] 		= empty  ,
["DATA"] 		= empty  ,
["POS"] 		= empty  ,
["TERR"] 		= empty  ,
["ADF1"] 		= empty  ,
["ADF2"] 		= empty  ,
["VOR1"] 		= empty  ,
["VOR2"] 		= empty  ,
}

-- ######################################################################### --


-- USER buttons for MODE 1
USER1 = {
["IDENT"]		= "Lght"  ,		-- 4 chars to be shown on dsp, when enter this mode

["0"] 			= Lights_PANEL_toggle  ,
["1"] 			= empty  ,
["2"] 			= empty  ,
["3"] 			= empty  ,
["4"] 			= empty  ,
["5"] 			= EFIS_MODE_toggle  ,
["6"] 			= MCP_MODE_toggle  ,
["7"] 			= USER_MODE_toggle  ,
}

-- USER buttons for MODE 2
USER2 = {
["IDENT"]		= "DeIc"  ,		-- 4 chars to be shown on dsp, when enter this mode

["0"] 			= empty  ,
["1"] 			= empty  ,
["2"] 			= empty  ,
["3"] 			= empty  ,
["4"] 			= empty  ,
["5"] 			= EFIS_MODE_toggle  ,
["6"] 			= MCP_MODE_toggle  ,
["7"] 			= USER_MODE_toggle  ,
}

-- USER buttons for MODE 3
USER3 = {
["ENABLED"]		= true  ,  		-- true / false to enable/disable MODE 3 for USER buttons
["IDENT"]		= "Othr"  ,		-- 4 chars to be shown on dsp, when enter this mode

["0"] 			= empty  ,
["1"] 			= empty  ,
["2"] 			= empty  ,
["3"] 			= empty  ,
["4"] 			= empty  ,
["5"] 			= EFIS_MODE_toggle  ,
["6"] 			= MCP_MODE_toggle  ,
["7"] 			= USER_MODE_toggle  ,
}

-- ######################################################################### --


-- RADIOS buttons ADDITIONAL actions
-- system hardencoded actions (if there any) will be executed anyway 	<<-- !!!
-- here you can just extend default functionality with additional actions
RADIOS = {
["COM1 Select"]	= empty  ,
["COM2 Select"]	= empty  ,
["COM1 Swap"]	= empty  ,
["COM2 Swap"]	= empty  ,
["COMs Mode"]	= Radios_COM_AUDIO_toggle  ,

["NAV1 Select"]	= empty  ,
["NAV2 Select"]	= empty  ,
["NAV1 Swap"]	= empty  ,
["NAV2 Swap"]	= empty  ,
["NAVs Mode"]	= Radios_NAV_AUDIO_toggle  ,

["ADF1 Select"]	= empty  ,
["ADF2 Select"]	= empty  ,
["ADFs Mode"]	= Radios_ADF_AUDIO_toggle  ,

["DME1 Select"]	= empty  ,
["DME2 Select"]	= empty  ,
["DMEs Mode"]	= Radios_DME_AUDIO_toggle  ,

["XPND Select"]	= empty  ,
["XPND Swap"]	= empty  ,
["XPND Mode"]	= TCAS_MODE_cycle  ,
}

-- ######################################################################### --

-- EFIS MINS knob for MCP MODE 1
MINS1 = {
["A SHOW"]		= empty  ,
["A +"]			= ECAM_MODE_next  ,
["A ++"]		= ECAM_MODE_next  ,
["A -"]			= ECAM_MODE_prev  ,
["A --"]		= ECAM_MODE_prev  ,

["PRESS"] 		= ECAM_TO_CONFIG  ,

["B SHOW"]		= empty  ,
["B +"]			= empty  ,
["B ++"]		= empty  ,
["B -"]			= empty  ,
["B --"]		= empty  ,
}


-- EFIS MINS knob for MCP MODE 2
MINS2 = {
["A SHOW"]		= empty  ,
["A +"]			= empty  ,
["A ++"]		= empty  ,
["A -"]			= empty  ,
["A --"]		= empty  ,

["PRESS"] 		= empty  ,

["B SHOW"]		= empty  ,
["B +"]			= empty  ,
["B ++"]		= empty  ,
["B -"]			= empty  ,
["B --"]		= empty  ,
}

-- EFIS MINS knob for MCP MODE 3
MINS3 = {
["A SHOW"]		= empty  ,
["A +"]			= empty  ,
["A ++"]		= empty  ,
["A -"]			= empty  ,
["A --"]		= empty  ,

["PRESS"] 		= empty  ,

["B SHOW"]		= empty  ,
["B +"]			= empty  ,
["B ++"]		= empty  ,
["B -"]			= empty  ,
["B --"]		= empty  ,
}

-- ######################################################################### --


-- EFIS BARO knob for MCP MODE 1
BARO1 = {
["A SHOW"]		= Altimeter_BARO_show  ,
["A +"]			= Altimeter_BARO_plus  ,
["A ++"]		= Altimeter_BARO_plusfast  ,
["A -"]			= Altimeter_BARO_minus  ,
["A --"]		= Altimeter_BARO_minusfast  ,

["PRESS"] 		= BARO_MODE_toggle  ,

["B SHOW"]		= empty  ,
["B +"]			= empty  ,
["B ++"]		= empty  ,
["B -"]			= empty  ,
["B --"]		= empty  ,
}

-- EFIS BARO knob for MCP MODE 2
BARO2 = {
["A SHOW"]		= empty  ,
["A +"]			= empty  ,
["A ++"]		= empty  ,
["A -"]			= empty  ,
["A --"]		= empty  ,

["PRESS"] 		= empty  ,

["B SHOW"]		= empty  ,
["B +"]			= empty  ,
["B ++"]		= empty  ,
["B -"]			= empty  ,
["B --"]		= empty  ,
}

-- EFIS BARO knob for MCP MODE 3
BARO3 = {
["A SHOW"]		= empty  ,
["A +"]			= empty  ,
["A ++"]		= empty  ,
["A -"]			= empty  ,
["A --"]		= empty  ,

["PRESS"] 		= empty  ,

["B SHOW"]		= empty  ,
["B +"]			= empty  ,
["B ++"]		= empty  ,
["B -"]			= empty  ,
["B --"]		= empty  ,
}

-- ######################################################################### --



-- EFIS CTR knob for MCP MODE 1
CTR1 = {
["A SHOW"]		= empty  ,
["A +"]			= ND_MODE_inc,
["A ++"]		= ND_MODE_inc  ,
["A -"]			= ND_MODE_dec  ,
["A --"]		= ND_MODE_dec  ,

["PRESS"] 		= KNOB_MODE_toggle  ,

["B SHOW"]		= empty  ,
["B +"]			= empty  ,
["B ++"]		= empty  ,
["B -"]			= empty  ,
["B --"]		= empty  ,
}


-- EFIS CTR knob for MCP MODE 2
CTR2 = {
["A SHOW"]		= empty  ,
["A +"]			= empty  ,
["A ++"]		= empty  ,
["A -"]			= empty  ,
["A --"]		= empty  ,

["PRESS"] 		= empty  ,

["B SHOW"]		= empty  ,
["B +"]			= empty  ,
["B ++"]		= empty  ,
["B -"]			= empty  ,
["B --"]		= empty  ,
}

-- EFIS CTR knob for MCP MODE 3
CTR3 = {
["A SHOW"]		= empty  ,
["A +"]			= empty  ,
["A ++"]		= empty  ,
["A -"]			= empty  ,
["A --"]		= empty  ,

["PRESS"] 		= empty  ,

["B SHOW"]		= empty  ,
["B +"]			= empty  ,
["B ++"]		= empty  ,
["B -"]			= empty  ,
["B --"]		= empty  ,
}

-- ######################################################################### --



-- EFIS TFC knob for MCP MODE 1
TFC1 = {
["A SHOW"]		= empty  ,
["A +"]			= ND_RANGE_inc  ,
["A ++"]		= ND_RANGE_inc  ,
["A -"]			= ND_RANGE_dec  ,
["A --"]		= ND_RANGE_dec  ,

["PRESS"] 		= KNOB_MODE_toggle  ,

["B SHOW"]		= empty  ,
["B +"]			= empty  ,
["B ++"]		= empty  ,
["B -"]			= empty  ,
["B --"]		= empty  ,
}

-- EFIS TFC knob for MCP MODE 2
TFC2 = {
["A SHOW"]		= empty  ,
["A +"]			= empty  ,
["A ++"]		= empty  ,
["A -"]			= empty  ,
["A --"]		= empty  ,

["PRESS"] 		= empty  ,

["B SHOW"]		= empty  ,
["B +"]			= empty  ,
["B ++"]		= empty  ,
["B -"]			= empty  ,
["B --"]		= empty  ,
}

-- EFIS TFC knob for MCP MODE 3
TFC3 = {
["A SHOW"]		= empty  ,
["A +"]			= empty  ,
["A ++"]		= empty  ,
["A -"]			= empty  ,
["A --"]		= empty  ,

["PRESS"] 		= empty  ,

["B SHOW"]		= empty  ,
["B +"]			= empty  ,
["B ++"]		= empty  ,
["B -"]			= empty  ,
["B --"]		= empty  ,
}
-- ######################################################################### --


-- MCP CRS knob for MCP MODE 1
CRS1 = {
["A SHOW"]		= CRS_show  ,
["A +"]			= CRS_plus  ,
["A ++"]		= CRS_plusfast  ,
["A -"]			= CRS_minus  ,
["A --"]		= CRS_minusfast  ,

["PRESS"] 		= empty  ,

["B SHOW"]		= empty  ,
["B +"]			= empty  ,
["B ++"]		= empty  ,
["B -"]			= empty  ,
["B --"]		= empty  ,
}


-- ######################################################################### --


-- MCP SPD knob for MCP MODE 1
SPD1 = {
["A SHOW"]		= SPD_show  ,
["A +"]			= SPD_plus  ,
["A ++"]		= SPD_plusfast  ,
["A -"]			= SPD_minus  ,
["A --"]		= SPD_minusfast  ,

["PRESS"] 		= Autopilot_SPD_toggle  ,

["B SHOW"]		= empty  ,
["B +"]			= empty  ,
["B ++"]		= empty  ,
["B -"]			= empty  ,
["B --"]		= empty  ,
}


-- ######################################################################### --


-- MCP HDG knob for MCP MODE 1
HDG1 = {
["A SHOW"]		= HDG_show  ,
["A +"]			= HDG_plus  ,
["A ++"]		= HDG_plusfast  ,
["A -"]			= HDG_minus  ,
["A --"]		= HDG_minusfast  ,

["PRESS"] 		= Autopilot_HDG_toggle  ,

["B SHOW"]		= empty  ,
["B +"]			= empty  ,
["B ++"]		= empty  ,
["B -"]			= empty  ,
["B --"]		= empty  ,
}



-- ######################################################################### --


-- MCP ALT knob for MCP MODE 1
ALT1 = {
["A SHOW"]		= ALT_show  ,
["A +"]			= ALT_plus  ,
["A ++"]		= ALT_plusfast  ,
["A -"]			= ALT_minus  ,
["A --"]		= ALT_minusfast  ,

["PRESS"] 		= Autopilot_ALT_toggle  ,

["B SHOW"]		= empty  ,
["B +"]			= empty  ,
["B ++"]		= empty  ,
["B -"]			= empty  ,
["B --"]		= empty  ,
}


-- ######################################################################### --


-- MCP V/S knob for MCP MODE 1
VVS1 = {
["A SHOW"]		= VVS_show  ,
["A +"]			= VVS_plus  ,
["A ++"]		= VVS_plusfast  ,
["A -"]			= VVS_minus  ,
["A --"]		= VVS_minusfast  ,

["PRESS"] 		= Autopilot_VS_level  ,

["B SHOW"]		= empty  ,
["B +"]			= empty  ,
["B ++"]		= empty  ,
["B -"]			= empty  ,
["B --"]		= empty  ,
}


-- ######################################################################### --