-- #############################################
-- ## VRIDRV Lua Universal Driver
-- ## PMDG JS41 Module, ver. 1b
-- #############################################


-- Default mode is MODE 1. 
-- If you dont need MODE 2, it's simple - just don't use it.


-- MCP buttons for MODE 1
MCP1 = {
["IDENT"]		= "MCP1"  ,		-- 4 chars to be shown on dsp, when enter this mode

["TOGA"] 		= Autopilot_TOGA_hold ,
["TOGN"] 		= Autopilot_TOGA_off ,
["N1"] 			= Autopilot_N1_hold ,
["SPD"] 		= Autopilot_IAS_hold ,
["FLCH"] 		= empty  ,
["HDGSEL"] 		= Autopilot_HDGSEL_hold ,
["HDGHLD"] 		= Autopilot_HDG_hold ,
["ALTHLD"] 		= Autopilot_ALT_hold ,
["V/S FPA"] 		= Autopilot_VS_hold ,
["APP"] 		= Autopilot_APR_hold ,
["VNAV"] 		= Autopilot_Attitude_hold ,
["LNAV"] 		= Autopilot_NAV_hold ,
["CMDA"] 		= Autopilot_MASTER_toggle,
["CMDB"] 		= empty  ,
["CMDC"] 		= empty  ,
["LOC"] 		= Autopilot_LOC_hold ,
["CWSA"] 		= Autopilot_NAVGPS_toggle ,
["CWSB"] 		= DSP_MODE_toggle,
["A/T UP"] 		= Autopilot_AT_arm ,
["A/T DN"] 		= Autopilot_AT_disarm ,
["F/D UP"] 		= Autopilot_FD_on ,
["F/D DN"] 		= Autopilot_FD_off ,
["MASTER UP"] 	= Autopilot_MASTER_on,
["MASTER DN"] 	= Autopilot_MASTER_off ,
}

-- MCP buttons for MODE 2
MCP2 = {
["IDENT"]		= "MCP2"  ,		-- 4 chars to be shown on dsp, when enter this mode

["TOGA"] 		= empty  ,
["TOGN"] 		= empty  ,
["N1"] 			= empty  ,
["SPD"] 		= empty  ,
["FLCH"] 		= empty  ,
["HDGSEL"] 		= empty  ,
["HDGHLD"] 		= empty  ,
["ALTHLD"] 		= empty  ,
["V/S FPA"] 	= empty  ,
["APP"] 		= empty  ,
["VNAV"] 		= empty  ,
["LNAV"] 		= empty  ,
["CMDA"] 		= empty  ,
["CMDB"] 		= empty  ,
["CMDC"] 		= empty  ,
["LOC"] 		= empty  ,
["CWSA"] 		= empty  ,
["CWSB"] 		= DSP_MODE_toggle,
["A/T UP"] 		= empty  ,
["A/T DN"] 		= empty  ,
["F/D UP"] 		= empty  ,
["F/D DN"] 		= empty  ,
["MASTER UP"] 	= empty  ,
["MASTER DN"] 	= empty  ,
}


-- MCP buttons for MODE 3
MCP3 = {
["ENABLED"]		= false  , 		-- true / false to enable/disable MODE 3 for MCP controls
["IDENT"]		= "MCP3"  ,		-- 4 chars to be shown on dsp, when enter this mode

["TOGA"] 		= empty  ,
["TOGN"] 		= empty  ,
["N1"] 			= empty  ,
["SPD"] 		= empty  ,
["FLCH"] 		= empty  ,
["HDGSEL"] 		= empty  ,
["HDGHLD"] 		= empty  ,
["ALTHLD"] 		= empty  ,
["V/S FPA"] 	= empty  ,
["APP"] 		= empty  ,
["VNAV"] 		= empty  ,
["LNAV"] 		= empty  ,
["CMDA"] 		= empty  ,
["CMDB"] 		= empty  ,
["CMDC"] 		= empty  ,
["LOC"] 		= empty  ,
["CWSA"] 		= empty  ,
["CWSB"] 		= DSP_MODE_toggle,
["A/T UP"] 		= empty  ,
["A/T DN"] 		= empty  ,
["F/D UP"] 		= empty  ,
["F/D DN"] 		= empty  ,
["MASTER UP"] 	= empty  ,
["MASTER DN"] 	= empty  ,
}

-- ######################################################################### --



-- EFIS buttons for MODE 1
EFIS1 = {
["IDENT"]		= "GPS"  ,		-- 4 chars to be shown on dsp, when enter this mode

["FPV"] 		= GPS_ENTER_Button ,
["MTR"] 		= GPS_DIRECTTO_Button ,
["WX"] 			= empty  ,
["STA"] 		= GPS_MENU_Button ,
["WPT"] 		= GPS_CLR_Button ,
["ARPT"] 		= GPS_NRST_Button ,
["DATA"] 		= GPS_FPL_Button ,
["POS"] 		= GPS_PROC_Button ,
["TERR"] 		= GPS_TERR_Button ,
["ADF1"] 		= EFIS_ADF1 ,
["ADF2"] 		= EFIS_ADF2 ,
["VOR1"] 		= EFIS_VOR1 ,
["VOR2"] 		= EFIS_VOR2 ,
}

-- EFIS buttons for MODE 2
EFIS2 = {
["IDENT"]		= "WXR"  ,		-- 4 chars to be shown on dsp, when enter this mode

["FPV"] 		= empty  ,
["MTR"] 		= empty  ,
["WX"] 			= empty  ,
["STA"] 		= empty  ,
["WPT"] 		= empty  ,
["ARPT"] 		= empty  ,
["DATA"] 		= empty  ,
["POS"] 		= empty  ,
["TERR"] 		= empty  ,
["ADF1"] 		= empty  ,
["ADF2"] 		= empty  ,
["VOR1"] 		= empty  ,
["VOR2"] 		= empty  ,
}

-- EFIS buttons for MODE 3
EFIS3 = {
["ENABLED"]		= false  ,  	-- true / false to enable/disable MODE 3 for EFIS controls
["IDENT"]		= "Othr"  ,		-- 4 chars to be shown on dsp, when enter this mode

["FPV"] 		= empty  ,
["MTR"] 		= empty  ,
["WX"] 			= empty  ,
["STA"] 		= empty  ,
["WPT"] 		= empty  ,
["ARPT"] 		= empty  ,
["DATA"] 		= empty  ,
["POS"] 		= empty  ,
["TERR"] 		= empty  ,
["ADF1"] 		= empty  ,
["ADF2"] 		= empty  ,
["VOR1"] 		= empty  ,
["VOR2"] 		= empty  ,
}

-- ######################################################################### --


-- USER buttons for MODE 1
USER1 = {
["IDENT"]		= "Lght"  ,		-- 4 chars to be shown on dsp, when enter this mode

["0"] 			= Lights_PANEL_toggle  ,
["1"] 			= Lights_LANDING_toggle ,
["2"] 			= empty  ,
["3"] 			= empty  ,
["4"] 			= empty  ,
["5"] 			= EFIS_MODE_toggle  ,
["6"] 			= MCP_MODE_toggle  ,
["7"] 			= USER_MODE_toggle  ,
}

-- USER buttons for MODE 2
USER2 = {
["IDENT"]		= "DeIc"  ,		-- 4 chars to be shown on dsp, when enter this mode

["0"] 			= empty  ,
["1"] 			= empty  ,
["2"] 			= empty  ,
["3"] 			= empty  ,
["4"] 			= empty  ,
["5"] 			= EFIS_MODE_toggle  ,
["6"] 			= MCP_MODE_toggle  ,
["7"] 			= USER_MODE_toggle  ,
}

-- USER buttons for MODE 3
USER3 = {
["ENABLED"]		= true  ,  		-- true / false to enable/disable MODE 3 for USER buttons
["IDENT"]		= "Othr"  ,		-- 4 chars to be shown on dsp, when enter this mode

["0"] 			= empty  ,
["1"] 			= empty  ,
["2"] 			= empty  ,
["3"] 			= empty  ,
["4"] 			= empty  ,
["5"] 			= EFIS_MODE_toggle  ,
["6"] 			= MCP_MODE_toggle  ,
["7"] 			= USER_MODE_toggle  ,
}

-- ######################################################################### --


-- RADIOS buttons ADDITIONAL actions
-- system hardencoded actions (if there any) will be executed anyway 	<<-- !!!
-- here you can just extend default functionality with additional actions
RADIOS = {
["COM1 Select"]	= empty  ,
["COM2 Select"]	= empty  ,
["COM1 Swap"]	= empty  ,
["COM2 Swap"]	= empty  ,
["COMs Mode"]	= Radios_COM_AUDIO_toggle  ,

["NAV1 Select"]	= empty  ,
["NAV2 Select"]	= empty  ,
["NAV1 Swap"]	= empty  ,
["NAV2 Swap"]	= empty  ,
["NAVs Mode"]	= Radios_NAV_AUDIO_toggle  ,

["ADF1 Select"]	= empty  ,
["ADF2 Select"]	= empty  ,
["ADFs Mode"]	= Radios_ADF_AUDIO_toggle  ,

["DME1 Select"]	= empty  ,
["DME2 Select"]	= empty  ,
["DMEs Mode"]	= Radios_DME_AUDIO_toggle  ,

["XPND Select"]	= empty  ,
["XPND Swap"]	= empty  ,
["XPND Mode"]	= Transponder_MODE_toggle  ,
}

-- ######################################################################### --

-- EFIS MINS knob for MCP MODE 1
MINS1 = {
["A SHOW"]		= GPS_PAGE_show ,
["A +"]			= GPS_PAGE_inc ,
["A ++"]		= GPS_PAGE_inc ,
["A -"]			= GPS_PAGE_dec ,
["A --"]		= GPS_PAGE_dec ,

["PRESS"] 		= KNOB_MODE_toggle  ,

["B SHOW"]		= EFIS_MINIMUMS_show ,
["B +"]			= EFIS_MINIMUMS_inc ,
["B ++"]		= EFIS_MINIMUMS_inc ,
["B -"]			= EFIS_MINIMUMS_dec ,
["B --"]		= EFIS_MINIMUMS_dec ,
}


-- EFIS MINS knob for MCP MODE 2
MINS2 = {
["A SHOW"]		= empty  ,
["A +"]			= empty  ,
["A ++"]		= empty  ,
["A -"]			= empty  ,
["A --"]		= empty  ,

["PRESS"] 		= empty  ,

["B SHOW"]		= empty  ,
["B +"]			= empty  ,
["B ++"]		= empty  ,
["B -"]			= empty  ,
["B --"]		= empty  ,
}

-- EFIS MINS knob for MCP MODE 3
MINS3 = {
["A SHOW"]		= empty  ,
["A +"]			= empty  ,
["A ++"]		= empty  ,
["A -"]			= empty  ,
["A --"]		= empty  ,

["PRESS"] 		= empty  ,

["B SHOW"]		= empty  ,
["B +"]			= empty  ,
["B ++"]		= empty  ,
["B -"]			= empty  ,
["B --"]		= empty  ,
}

-- ######################################################################### --


-- EFIS BARO knob for MCP MODE 1
BARO1 = {
["A SHOW"]		= Altimeter_BARO_show  ,
["A +"]			= Altimeter_BARO_plus  ,
["A ++"]		= Altimeter_BARO_plusfast  ,
["A -"]			= Altimeter_BARO_minus  ,
["A --"]		= Altimeter_BARO_minusfast  ,

["PRESS"] 		= Altimeter_BARO_MODE_toggle  ,

["B SHOW"]		= empty  ,
["B +"]			= empty  ,
["B ++"]		= empty  ,
["B -"]			= empty  ,
["B --"]		= empty  ,
}

-- EFIS BARO knob for MCP MODE 2
BARO2 = {
["A SHOW"]		= empty  ,
["A +"]			= empty  ,
["A ++"]		= empty  ,
["A -"]			= empty  ,
["A --"]		= empty  ,

["PRESS"] 		= empty  ,

["B SHOW"]		= empty  ,
["B +"]			= empty  ,
["B ++"]		= empty  ,
["B -"]			= empty  ,
["B --"]		= empty  ,
}

-- EFIS BARO knob for MCP MODE 3
BARO3 = {
["A SHOW"]		= empty  ,
["A +"]			= empty  ,
["A ++"]		= empty  ,
["A -"]			= empty  ,
["A --"]		= empty  ,

["PRESS"] 		= empty  ,

["B SHOW"]		= empty  ,
["B +"]			= empty  ,
["B ++"]		= empty  ,
["B -"]			= empty  ,
["B --"]		= empty  ,
}

-- ######################################################################### --



-- EFIS CTR knob for MCP MODE 1
CTR1 = {
["A SHOW"]		= empty,
["A +"]			= GPS_GROUP_inc ,
["A ++"]		= GPS_GROUP_inc ,
["A -"]			= GPS_GROUP_dec ,
["A --"]		= GPS_GROUP_dec ,

["PRESS"] 		= GPS_CRSR_Button ,

["B SHOW"]		= empty,
["B +"]			= empty,
["B ++"]		= empty,
["B -"]			= empty,
["B --"]		= empty,
}


-- EFIS CTR knob for MCP MODE 2
CTR2 = {
["A SHOW"]		= empty  ,
["A +"]			= empty  ,
["A ++"]		= empty  ,
["A -"]			= empty  ,
["A --"]		= empty  ,

["PRESS"] 		= empty  ,

["B SHOW"]		= empty  ,
["B +"]			= empty  ,
["B ++"]		= empty  ,
["B -"]			= empty  ,
["B --"]		= empty  ,
}

-- EFIS CTR knob for MCP MODE 3
CTR3 = {
["A SHOW"]		= empty  ,
["A +"]			= empty  ,
["A ++"]		= empty  ,
["A -"]			= empty  ,
["A --"]		= empty  ,

["PRESS"] 		= empty  ,

["B SHOW"]		= empty  ,
["B +"]			= empty  ,
["B ++"]		= empty  ,
["B -"]			= empty  ,
["B --"]		= empty  ,
}

-- ######################################################################### --



-- EFIS TFC knob for MCP MODE 1
TFC1 = {
["A SHOW"]		= empty  ,
["A +"]			= GPS_Zoom_inc ,
["A ++"]		= GPS_Zoom_inc ,
["A -"]			= GPS_Zoom_dec ,
["A --"]		= GPS_Zoom_dec ,

["PRESS"] 		= GPS_CRSR_Button ,

["B SHOW"]		= empty  ,
["B +"]			= empty  ,
["B ++"]		= empty  ,
["B -"]			= empty  ,
["B --"]		= empty  ,
}

-- EFIS TFC knob for MCP MODE 2
TFC2 = {
["A SHOW"]		= empty  ,
["A +"]			= empty  ,
["A ++"]		= empty  ,
["A -"]			= empty  ,
["A --"]		= empty  ,

["PRESS"] 		= empty  ,

["B SHOW"]		= empty  ,
["B +"]			= empty  ,
["B ++"]		= empty  ,
["B -"]			= empty  ,
["B --"]		= empty  ,
}

-- EFIS TFC knob for MCP MODE 3
TFC3 = {
["A SHOW"]		= empty  ,
["A +"]			= empty  ,
["A ++"]		= empty  ,
["A -"]			= empty  ,
["A --"]		= empty  ,

["PRESS"] 		= empty  ,

["B SHOW"]		= empty  ,
["B +"]			= empty  ,
["B ++"]		= empty  ,
["B -"]			= empty  ,
["B --"]		= empty  ,
}
-- ######################################################################### --


-- MCP CRS knob for MCP MODE 1
CRS1 = {
["A SHOW"]		= CRS_show  ,
["A +"]			= CRS_plus  ,
["A ++"]		= CRS_plusfast  ,
["A -"]			= CRS_minus  ,
["A --"]		= CRS_minusfast  ,

["PRESS"] 		= empty  ,

["B SHOW"]		= empty  ,
["B +"]			= empty  ,
["B ++"]		= empty  ,
["B -"]			= empty  ,
["B --"]		= empty  ,
}


-- ######################################################################### --


-- MCP SPD knob for MCP MODE 1
SPD1 = {
["A SHOW"]		= SPD_show  ,
["A +"]			= SPD_plus  ,
["A ++"]		= SPD_plusfast  ,
["A -"]			= SPD_minus  ,
["A --"]		= SPD_minusfast  ,

["PRESS"] 		= empty  ,

["B SHOW"]		= empty  ,
["B +"]			= empty  ,
["B ++"]		= empty  ,
["B -"]			= empty  ,
["B --"]		= empty  ,
}


-- ######################################################################### --


-- MCP HDG knob for MCP MODE 1
HDG1 = {
["A SHOW"]		= HDG_show  ,
["A +"]			= HDG_plus  ,
["A ++"]		= HDG_plusfast  ,
["A -"]			= HDG_minus  ,
["A --"]		= HDG_minusfast  ,

["PRESS"] 		= Autopilot_HDG_Bug_align ,

["B SHOW"]		= empty  ,
["B +"]			= empty  ,
["B ++"]		= empty  ,
["B -"]			= empty  ,
["B --"]		= empty  ,
}

-- ######################################################################### --


-- MCP ALT knob for MCP MODE 1
ALT1 = {
["A SHOW"]		= ALT_show  ,
["A +"]			= ALT_plus  ,
["A ++"]		= ALT_plusfast  ,
["A -"]			= ALT_minus  ,
["A --"]		= ALT_minusfast  ,

["PRESS"] 		= Autopilot_ALTSEL_mode  ,

["B SHOW"]		= empty  ,
["B +"]			= empty  ,
["B ++"]		= empty  ,
["B -"]			= empty  ,
["B --"]		= empty  ,
}


-- ######################################################################### --


-- MCP V/S knob for MCP MODE 1
VVS1 = {
["A SHOW"]		= VVS_show  ,
["A +"]			= VVS_plus  ,
["A ++"]		= VVS_plusfast  ,
["A -"]			= VVS_minus  ,
["A --"]		= VVS_minusfast  ,

["PRESS"] 		= Autopilot_VSSEL_mode ,

["B SHOW"]		= empty  ,
["B +"]			= empty  ,
["B ++"]		= empty  ,
["B -"]			= empty  ,
["B --"]		= empty  ,
}

-- ######################################################################### --