-- ## System functions ###############
-- universal toggler storage
tgl = {}
ipcPARAM = 0

-- functions switcher
function switch (command, actions, group, knob_command)

	local f = actions [command]
	
	if type (f) ~= "function" then
	
		if f == nil then return false end
	
		if string.find(f, "Keys:") ~= nil then
		
			KeyPress (string.sub(f, 7))
		
			return true
		
		end
	
		if string.find(f, "Control:") ~= nil then
		
			FSXcontrol (string.sub(f, 10))
		
			return
		
		end

		if string.find(f, "FSX:") ~= nil then
		
			FSXcontrolName (string.sub(f, 6))
		
			return
		
		end
		
		-- _log(f)
		
		if string.sub(f, 1, 2) == "M:" then
		
			FSUIPCmacro (string.sub(f, 4))

			return
		
		end

		_logg("Handle not found! " .. ipc.get("acft_handle") .. " - " .. command .. " / " .. group)
		return false
		
	else

    	-- _log('switch: [' .. command .. ']')

		f (command, group, knob_command)
	
		return true
		
	end -- if

end -- switch

-- rounds the integer
function round(num)
	num = tonumber(num)
	if num == nil then return 0 end
    if num >= 0 then return math.floor(num+.5) 
    else return math.ceil(num-.5) end
end

function round2 (num, idp)
  num = tonumber(num)
  if num == nil then return 0 end
  local mult = 10^(idp or 0)
  return math.floor(num * mult + 0.5) / mult
end

-- debug level 1
function _err (s)
	ipc.log("LINDA:: ---> ERROR:: " .. s)
end

-- debug level 1
function _log (s)
	if ipc.get("DEBUG") > 0 then ipc.log("LINDA:: " .. s) end
end

-- debug level 2
function _logg (s)
	if ipc.get("DEBUG") > 1 then ipc.log("LINDA:: " .. s) end
end

function _logf (func, command)

	if ipc.get("DEBUG") > 0 then 
		ipc.log("LINDA: function call: " .. func .. "/" .. tostring(command))
	end

end

function _logff (func, command)

	if ipc.get("DEBUG") > 1 then 
		ipc.log("LINDA: function call: " .. func .. "/" .. tostring(command))
	end

end

-- returns the 'basename' and the 'basedir' 
function getBase(filename) 
    return  string.gsub(filename, "(.*/)(.*)", "%2") or filename
end

-- check file exist
function file_exists(name)
   local f=io.open(name,"r")
   if f~=nil then io.close(f) return true else return false end
end

function Hex2Bin(s)

	local hex2bin = {
		["0"] = "0000",
		["1"] = "0001",
		["2"] = "0010",
		["3"] = "0011",
		["4"] = "0100",
		["5"] = "0101",
		["6"] = "0110",
		["7"] = "0111",
		["8"] = "1000",
		["9"] = "1001",
		["a"] = "1010",
		["b"] = "1011",
		["c"] = "1100",
		["d"] = "1101",
		["e"] = "1110",
		["f"] = "1111"
		}

	-- s	-> hexadecimal string

	local ret = ""
	local i = 0


	for i in string.gfind(s, ".") do
		i = string.lower(i)

		ret = ret..hex2bin[i]

	end

	return ret
end

-- string split
function split(str, pat)
   str = tostring(str)
   local t = {}  -- NOTE: use {n = 0} in Lua-5.0
   local fpat = "(.-)" .. pat
   local last_end = 1
   local s, e, cap = str:find(fpat, 1)
   while s do
      if s ~= 1 or cap ~= "" then
	  table.insert(t,cap)
      end
      last_end = e+1
      s, e, cap = str:find(fpat, last_end)
   end
   if last_end <= #str then
      cap = str:sub(last_end)
      table.insert(t, cap)
   end
   return t
end

-- table entries counter
function tablelength(T)
  local count = 0
  for _ in pairs(T) do count = count + 1 end
  return count
end

-- random sleep 
function _sleep(from, to)

	if to ~= nil then
	
		ipc.sleep(math.random(from, to))
		
	else
	
		ipc.sleep(from)
	
	end

end

-- universal toggler for any actions
function _t (command)

	if tgl[command] == nil then
	
		tgl[command] = true
		return true
		
	end
	
	tgl[command] = nil
	return false

end


-- universal toggler by LVar
function _tt (lvar, compare)

	if compare == nil then compare = 0 end

	if ipc.readLvar(lvar) == compare then

		return true

	end

	return false

end

-- universal toggler by LVar
function _tl (lvar, compare)

    return _tt (lvar, compare)

end


-- universal toggler by value
function _tv (var, compare)

	if var == compare then

		return true

	end

	return false

end



-- empty command handler -- DO NOT DELETE!
function empty (command, group)

	if group == "MCP" then

		_log("MCP MODE " .. ipc.get("MCPmode") .. ": Empty command for :: " .. command)
		
	elseif group == "EFIS" then

		_log("EFIS MODE " .. ipc.get("EFISmode") .. ": Empty command for :: " .. command)
		
	elseif group == "USER" then
	
		_log("USER MODE " .. ipc.get("USERmode") .. ": Empty command for :: " .. command)
		
	elseif group == "RADIOS" then
	
		_log("RADIOS : Empty command for :: " .. command)

	else
		
		_logg("Empty call")

	end
	
end

-- ################### --


-- ### INIT COMMON VARS ##############
function CommonInitVars ()

    -- MCP 2 vars -------------------

    SPD = 100
    HDG = 0
    ALT = 100
    VVS = 0
    OBS1 = 999
    OBS2 = 999
    SPDfast = 0
    HDGfast = 0
    ALTfast = 0
    VVSfast = 0
    AutoDisplay = true

    RADIOS_MODE = 0 -- 1 com, 2 nav, 3 adf, 4 dme, 5 xpnd
    RADIOS_SUBMODE = 2 --
    RADIOS_MhzKhz = 1 -- 1 Mhz, 2 KHz

    RADIOS_CURSOR_POS = 3 -- adf cursor position

    RADIOS_ADF_FREQ1 = ''
    RADIOS_ADF_FREQ2 = ''

    RADIOS_DME_CRS1 = 0
    RADIOS_DME_CRS2 = 0

    RADIOS_XPND_CODE = ''

    RADIOS_MSG = false -- true on message show, used for auto-clear
    RADIOS_MSG_SHORT = false

    AP_STATE = -1

    DSP_MSG = false

    DSP_MSG_PREV1 = ''
    DSP_MSG_PREV2 = ''

    FLIGHT_INFO1 = ''
    FLIGHT_INFO2 = ''

    if _MCP1() then

        FLIGHT_INFO_TEXT = 'INFO'

    else

        FLIGHT_INFO_TEXT = ' INFO '

    end

    -- ENDOF MCP 2 vars -------------

	-- init or not yet
	init = false
	
	dev = 0

	com_sel = 1
	com_open = 0
	com1_firstload = true
	com2_firstload = true
	com1x_freq = ipc.readUW(0x034E)
	com1s_freq = ipc.readUW(0x311A)
	com2x_freq = ipc.readUW(0x3118)
	com2s_freq = ipc.readUW(0x311C)
	com_audio = 1
	

	nav_sel = 1
	nav_open = 0
	nav1_firstload = true
	nav2_firstload = true
	nav1x_freq = ipc.readUW(0x0350)
	nav1s_freq = ipc.readUW(0x311E)
	nav2x_freq = ipc.readUW(0x3120)
	nav2s_freq = ipc.readUW(0x311C)
	nav_audio = 1

	adf_sel = 1
	adf1_firstload = true
	adf2_firstload = true

	trn_firstload = true

	dme_sel = 1
	dme_open = 0

	-- sync dsp
	sync_spd = 0
	sync_hdg = 0
	sync_alt = 0
	
	
	-- Starting MCP modes
	ipc.set("GLOBmode", 1)
	ipc.set("EFISmode", 1) 
	ipc.set("MCPmode", 1) 
	ipc.set("USERmode", 1) 
	
	-- Starting knob modes
	MINSmode = "A"
	BAROmode = "A"
	CTRmode = "A"
	TFCmode = "A"
	
	CRSmode = "A"
	SPDmode = "A"
	HDGmode = "A"
	ALTmode = "A"
	VVSmode = "A"

	baro_mode = 0 	-- initial borometer mode hPa
	baro_cur = 0		-- current baro
	fast_baro = 32 	-- fast rotation increment
	
	-- Display mode
	-- 1: autopilot
	-- 2: current
	ipc.set("DSPmode", 1) 
	
	ipc.set("FIP", 0)
	
	dsp_count = 0 -- counter for DSP auto-clear feature
	dsp0_prev = "" -- vars to save previos DSP state for auto-clear feature
	dsp1_prev = ""
	
	math.randomseed( os.time() )
	
	ipc.set("APlock", 0)
	
	-- Joystick init
	hidinit = false
	jdev = {}
	rd = {}
	wrf = {}
	wr = {}
	init1 = {}
	CurrentData = {}
	n = {}
	jcount = 1
	buttons = {}
	prevbuttons = {} 
	newval = {}
	prevval = {}	

	J={}
	
	JSTK={}
	JSTKrp={}
	JSTKrl={}

	JSTK2={}
	JSTK2rp={}
	JSTK2rl={}
	
	JREP={} -- repeat rate (not used yet)
	
	JDEVID={}
	
	JHATS={} -- hats HEX identifiers
	
	SHIFT_GLOB = 0
	SHIFT_LOC = {}
	
	-- keypress tables init
	KeyPressInit ()
	
	HUNTER = true
	
end
	
-- #######################################################

-- ## Device types ###############

function _MCP1 ()

    if ipc.get("VRItype") == 'mcp' then

        return true

    else

        return false

    end

end

function _MCP2 ()

    if ipc.get("VRItype") == 'mcp2' then

        return true

    else

        return false

    end

end

-- ## Special controls ##############

-- ### KeyPress sender ##############
function KeyPressInit ()

	MT = {}
	MT["-"] = 0
	MT["Ctrl"] = 10
	MT["Alt"] = 16
	MT["Shift"] = 9
	MT["Ctrl+Alt"] = 18
	MT["Ctrl+Shift"] = 11
	MT["Shift+Alt"] = 17
	MT["Ctrl+Shift+Alt"] = 19
	
	KT = {}
	KT["Pause"] =              19
	KT["Escape"] =             27
	KT["PgUp"] =            33
	KT["PgDn"] =          34
	KT["End"] =                35
	KT["Home"] =               36
	KT["Left"] =               37
	KT["Up"] =                 38
	KT["Right"] =              39
	KT["Down"] =               40
	KT["PrintScreen"] =        44
	KT["Insert"] =             45
	KT["Delete"] =             46
	KT["0"] =                  48
	KT["1"] =                  49
	KT["2"] =                  50
	KT["3"] =                  51
	KT["4"] =                  52
	KT["5"] =                  53
	KT["6"] =                  54
	KT["7"] =                  55
	KT["8"] =                  56
	KT["9"] =                  57
	KT["A"] =                  65
	KT["B"] =                  66
	KT["C"] =                  67
	KT["D"] =                  68
	KT["E"] =                  69
	KT["F"] =                  70
	KT["G"] =                  71
	KT["H"] =                  72
	KT["I"] =                  73
	KT["J"] =                  74
	KT["K"] =                  75
	KT["L"] =                  76
	KT["M"] =                  77
	KT["N"] =                  78
	KT["O"] =                  79
	KT["P"] =                  80
	KT["Q"] =                  81
	KT["R"] =                  82
	KT["S"] =                  83
	KT["T"] =                  84
	KT["U"] =                  85
	KT["V"] =                  86
	KT["W"] =                  87
	KT["X"] =                  88
	KT["Y"] =                  89
	KT["Z"] =                  90
	KT["Num 0"] =              96
	KT["Num 1"] =              97
	KT["Num 2"] =              98
	KT["Num 3"] =              99
	KT["Num 4"] =             100
	KT["Num 5"] =             101
	KT["Num 6"] =             102
	KT["Num 7"] =             103
	KT["Num 8"] =             104
	KT["Num 9"] =             105
	KT["Num *"] =             106
	KT["Num +"] =             107
	KT["Num -"] =             109
	KT["Num ."] =             110
	KT["Num /"] =             111
	KT["F1"] =                112
	KT["F2"] =                113
	KT["F3"] =                114
	KT["F4"] =                115
	KT["F5"] =                116
	KT["F6"] =                117
	KT["F7"] =                118
	KT["F8"] =                119
	KT["F9"] =                120
	KT["F10"] =               121
	KT["F11"] =               122
	KT["F12"] =               123
	KT["F13"] =               124
	KT["F14"] =               125
	KT["F15"] =               126
	KT["F16"] =               127
	KT["F17"] =               128
	KT["F18"] =               129
	KT["F19"] =               130
	KT["F20"] =               131
	KT["F21"] =               132
	KT["F22"] =               133
	KT["F23"] =               134
	KT["Num Lock"] =          144
	KT["Scroll Lock"] =       145
	KT[";"] =                 186
	KT["="] =                 187
	KT[","] =                 188
	KT["-"] =                 189
	KT["."] =                 190
	KT["/"] =                 191
	KT["'"] =                 192
	KT["["] =                 219
	KT["\\"] =                220
	KT["]"] =                 221
	KT["#"] =                 222
	KT["`"] =                 223

end;

function KeyPress (shortcut)

	local modifiers = '-'
	local modcode = 0
	local kecode = 0

	if string.find(shortcut, '+') ~= nil then

		shortcut = string.reverse(shortcut)
		
		modifiers = string.sub(shortcut, string.find(shortcut, '+') + 1)
		
		modifiers = string.reverse(modifiers)
		
		shortcut = string.sub(shortcut, 1, string.find(shortcut, '+') - 1)
		
		shortcut = string.reverse(shortcut)
		
	end
	
	modcode = MT[modifiers]
	keycode = KT[shortcut]
		
	ipc.keypress(keycode, modcode)
	
	_logg ("Keypress: " .. modifiers .. " : " .. shortcut)

end
-- #######################################################

-- ### FSUIPC macro exec #######
function FSUIPCmacro (command)
--[[
	local param = 0

	if string.find(command, '#') then

		param = tonumber(string.sub(command, string.find(command, '#') + 2))
		command =  string.sub(command, 1, string.find(command, '#') - 1)
	
		if param == nil then param = 0 end
	
		ipc.macro(command, tonumber(param))
		
		return
		
	end

--]]	

--    command = 'J41: AP_CRS plus'

	ipc.macro(command)
	
	_log('FSUIPC Macro request: [' .. command .. ']')

end
-- #######################################################

-- ### FSX control sender ##############
function FSXcontrol (control)

	local param = string.sub(control, string.find(control, ':') + 1)
	control =  string.sub(control, 1, string.find(control, ':') - 1)
	
	param = tonumber(param)
	control = tonumber(control)

	if control > 0 then
		
		if control == 65733 then
		
			_err("FSX control 65733 (ABORT) called. Skipping...")
		
			return
			
		end

		ipc.control(control, param)
	
	end
	
	_logg ("FSX control: " .. tostring(control) .. "  param: " .. tostring(param))

end

function FSXcontrolName (control)

	local param = string.sub(control, string.find(control, ':') + 1)
	control =  string.sub(control, 1, string.find(control, ':') - 1)
	
	param = tonumber(param)

	if type(_G['_' .. control]) == 'function' then
	
		_G['_' .. control](param)
	
	end
	
	_logg ("FSX control by name: " .. control .. "  param: " .. tostring(param))

end
-- #######################################################


-- ## RADIOS COMMON ###############

function Default_RADIOS_press ()

    if RADIOS_MODE == 1 or RADIOS_MODE == 2 then

        if RADIOS_MhzKhz == 1 then

            RADIOS_MhzKhz = 2
        	ipc.sleep(50)
	        com.write(dev, 'RADKhz', 8)

        else

            RADIOS_MhzKhz = 1
        	ipc.sleep(50)
	        com.write(dev, 'RADMhz', 8)

        end

    elseif RADIOS_MODE == 3 then

        RADIOS_CURSOR_POS = RADIOS_CURSOR_POS - 1

        if RADIOS_CURSOR_POS < 0 then RADIOS_CURSOR_POS = 3 end

       	ipc.sleep(50)
        com.write(dev, 'RADA' .. tostring(RADIOS_CURSOR_POS), 8)

    elseif RADIOS_MODE == 4 then

        -- Swap DME1 / DME2
        Default_DME_select ()

        -- Calling extended action
        switch ("PRESS", DME1, "RADIOS/DME")

        --[[
        if RADIOS_MODE == 1 then

            -- drop CRS1 to zero
            ipc.writeUW(0x0C4E, 0)

        else

            -- drop CRS2 to zero
            ipc.writeUW(0x0C5E, 0)

        end
        --]]

    elseif RADIOS_MODE == 5 then

        RADIOS_CURSOR_POS = RADIOS_CURSOR_POS - 1

        if RADIOS_CURSOR_POS < 0 then RADIOS_CURSOR_POS = 3 end

       	ipc.sleep(50)
        com.write(dev, 'RADT' .. tostring(RADIOS_CURSOR_POS), 8)

    end

end

function Default_RADIOS_swap ()

    local buffer_s, buffer_x, mhz, khz

    if RADIOS_MODE == 1 then

        if RADIOS_SUBMODE == 1 then

            -- COM1
            Default_COM_1_swap ()
	        buffer_x = string.format("%04x", ipc.readUW(0x034E))
	        buffer_s = string.format("%04x", ipc.readUW(0x311A))
        	ipc.sleep(20)
        	com.write(dev, 'COMx' .. buffer_s, 8)
        	ipc.sleep(20)
        	com.write(dev, 'COMs' .. buffer_x, 8)

   	    	switch ("COM1 Swap", RADIOS, "RADIOS")

        else

            -- COM2
            Default_COM_2_swap ()
	        buffer_x = string.format("%04x", ipc.readUW(0x3118))
	        buffer_s = string.format("%04x", ipc.readUW(0x311C))
        	ipc.sleep(20)
        	com.write(dev, 'COMX' .. buffer_s, 8)
        	ipc.sleep(20)
        	com.write(dev, 'COMS' .. buffer_x, 8)

   	    	switch ("COM2 Swap", RADIOS, "RADIOS")

        end

        DspRadioMed('<->')
       	com.write(dev, 'RADMhz', 8)


    elseif RADIOS_MODE == 2 then

        if RADIOS_SUBMODE == 1 then

            -- NAV1
            Default_NAV_1_swap ()
	        buffer_x = string.format("%04x", ipc.readUW(0x0350))
	        buffer_s = string.format("%04x", ipc.readUW(0x311E))
        	ipc.sleep(20)
        	com.write(dev, 'NAVx' .. buffer_s, 8)
        	ipc.sleep(20)
        	com.write(dev, 'NAVs' .. buffer_x, 8)

   	    	switch ("NAV1 Swap", RADIOS, "RADIOS")

        else

           -- NAV2
            Default_NAV_2_swap ()
	        buffer_x = string.format("%04x", ipc.readUW(0x0352))
	        buffer_s = string.format("%04x", ipc.readUW(0x3120))
        	ipc.sleep(20)
        	com.write(dev, 'NAVX' .. buffer_s, 8)
        	ipc.sleep(20)
        	com.write(dev, 'NAVS' .. buffer_x, 8)

   	    	switch ("NAV2 Swap", RADIOS, "RADIOS")

        end

        DspRadioMed('<->')
       	com.write(dev, 'RADMhz', 8)

    elseif RADIOS_MODE == 3 then


    elseif RADIOS_MODE == 4 then


    elseif RADIOS_MODE == 5 then

        Default_XPND_set (RADIOS_XPND_CODE)
    	switch ("XPND Swap", RADIOS, "RADIOS")

    end

end


function Default_RADIOS_mode ()

    if RADIOS_MODE == 1 then

		switch ("COMs Mode", RADIOS, "RADIOS")

    elseif RADIOS_MODE == 2 then

		switch ("NAVs Mode", RADIOS, "RADIOS")

    elseif RADIOS_MODE == 3 then

		switch ("ADFs Mode", RADIOS, "RADIOS")

    elseif RADIOS_MODE == 4 then

		switch ("DMEs Mode", RADIOS, "RADIOS")

    elseif RADIOS_MODE == 5 then

		switch ("XPND Mode", RADIOS, "RADIOS")

    end

end


function Default_RADIOS_plus ()

    if RADIOS_MODE == 1 then

        Default_NAVCOM_plus (0)

    elseif RADIOS_MODE == 2 then

        Default_NAVCOM_plus (1)

    elseif RADIOS_MODE == 3 then

        Default_ADF_plus ()

    elseif RADIOS_MODE == 4 then

        Default_DME_plus (false)

        -- Calling extended action
        switch ("A  +", DME1, "RADIOS/DME")

    elseif RADIOS_MODE == 5 then

        Default_XPND_plus ()

    end

end

function Default_RADIOS_plusfast ()

    if RADIOS_MODE == 1 then

        Default_NAVCOM_plus (0)

    elseif RADIOS_MODE == 2 then

        Default_NAVCOM_plus (1)

    elseif RADIOS_MODE == 3 then

        Default_ADF_plus ()

    elseif RADIOS_MODE == 4 then

        Default_DME_plus (true)

        -- Calling extended action
        switch ("A ++", DME1, "RADIOS/DME")

    elseif RADIOS_MODE == 5 then

        Default_XPND_plus ()

    end

end

function Default_RADIOS_minus ()

    if RADIOS_MODE == 1 then

        Default_NAVCOM_minus (0)

    elseif RADIOS_MODE == 2 then

        Default_NAVCOM_minus (1)

    elseif RADIOS_MODE == 3 then

        Default_ADF_minus ()

    elseif RADIOS_MODE == 4 then

        Default_DME_minus (false)

        -- Calling extended action
        switch ("A  -", DME1, "RADIOS/DME")

    elseif RADIOS_MODE == 5 then

        Default_XPND_minus ()

    end

end

function Default_RADIOS_minusfast ()

    if RADIOS_MODE == 1 then

        Default_NAVCOM_minus (0)

    elseif RADIOS_MODE == 2 then

        Default_NAVCOM_minus (1)

    elseif RADIOS_MODE == 3 then

        Default_ADF_minus ()

    elseif RADIOS_MODE == 4 then

        Default_DME_minus (true)

        -- Calling extended action
        switch ("A --", DME1, "RADIOS/DME")

    elseif RADIOS_MODE == 5 then

        Default_XPND_minus ()

    end

end



-- ## DME ##############

function Default_DME_select ()

    if RADIOS_MODE ~= 4 then

        RADIOS_SUBMODE = 2

    end

    RADIOS_MODE = 4 -- DME

    if RADIOS_SUBMODE == 2 then

    	ipc.sleep(50)
        com.write(dev, "DMExxxxx", 8)

        Default_DME_1_init (true)
        RADIOS_SUBMODE = 1

   		switch ("DME1 Select", RADIOS, "RADIOS")

    else

    	ipc.sleep(50)
        com.write(dev, "DMEXXXXX", 8)

        Default_DME_2_init (true)
        RADIOS_SUBMODE = 2

   		switch ("DME2 Select", RADIOS, "RADIOS")

    end

end


function Default_DME_set (s)
	
	s = tonumber(string.sub(s, 1, 3))

	_logg("SetDME " .. dme_sel .. " : " .. s)

	if dme_sel == 1 then
		ipc.writeUW(0x0C4E, s)
	else
		ipc.writeUW(0x0C5E, s)
	end
	
end

function Default_DME_1_init ()

	if dev == 0 then return end

    local vor_name1 = string.format("%s", ipc.readSTR(0x3000, 5))

    while string.len(vor_name1) < 5 do vor_name1 = vor_name1 .. ' ' end

    RADIOS_DME_CRS1 = ipc.readUW(0x0C4E)

    if not RADIOS_MSG_SHORT then

        com.write(dev, "dsp0DME\\", 8)
        ipc.sleep(20)

    end

    local vor_crs1 = string.format("%003d", RADIOS_DME_CRS1)

	if vor_name1 ~= "" then

		local vor_distance1 = tonumber(ipc.readUW(0x0300))
		local vor_speed1 = tonumber(ipc.readUW(0x0302)) / 10 

		vor_distance1 = string.format("DMEd%04d", vor_distance1)
		vor_speed1 = string.format("DMEs%03d", vor_speed1)


        if _MCP2() then

            while string.len(vor_name1) < 8 do vor_name1 = " " .. vor_name1 end

            com.write(dev, "dsp1" .. string.sub(vor_name1, 1, 4), 8)
            ipc.sleep(50)
            com.write(dev, "dsp2" .. string.sub(vor_name1, 5, 8), 8)
            ipc.sleep(50)

            if OBS1 == 999 then

                com.write(dev, "dsp3/" .. vor_crs1, 8)
                ipc.sleep(50)

            else

                com.write(dev, "dsp3/" .. string.format("%003d", OBS1), 8)
                ipc.sleep(50)

            end

        else

    	   	com.write(dev, "DMi" .. vor_name1 , 8)

        end

		com.write(dev, vor_distance1, 8)
		ipc.sleep(50)
		com.write(dev, vor_speed1, 8)
		ipc.sleep(50)

    else

        if OBS1 == 999 then

            com.write(dev, "dsp2  --", 8)
            ipc.sleep(50)
            com.write(dev, "dsp3/" .. vor_crs1, 8)
            ipc.sleep(50)

        else

            com.write(dev, "dsp2  --", 8)
            ipc.sleep(50)
            com.write(dev, "dsp3/" .. string.format("%003d", OBS1), 8)
            ipc.sleep(50)

        end

	end	
	
	dme_sel = 1
	dme_open = 1
	com_open = 0
	nav_open = 0
	
end

function Default_DME_2_init ()

	if dev == 0 then return end

    local vor_name2 = string.format("%s", ipc.readSTR(0x301F, 5))

    while string.len(vor_name2) < 5 do vor_name2 = vor_name2 .. ' ' end

    RADIOS_DME_CRS2 = ipc.readUW(0x0C5E)

    if not RADIOS_MSG_SHORT then

        com.write(dev, "dsp0DME\\", 8)
        ipc.sleep(20)

    end

    local vor_crs2 = string.format("%003d", RADIOS_DME_CRS2)

	if vor_name2 ~= "" then

		local vor_distance2 = tonumber(ipc.readUW(0x0306))
		local vor_speed2 = tonumber(ipc.readUW(0x0308)) / 10

		vor_distance2 = string.format("DMED%04d", vor_distance2)
		vor_speed2 = string.format("DMES%03d ", vor_speed2)

        if _MCP2() then

            while string.len(vor_name2) < 8 do vor_name2 = " " .. vor_name2 end

            com.write(dev, "dsp1" .. string.sub(vor_name2, 1, 4), 8)
            ipc.sleep(50)
            com.write(dev, "dsp2" .. string.sub(vor_name2, 5, 8), 8)
            ipc.sleep(50)

            if OBS2 == 999 then

                com.write(dev, "dsp3/" .. vor_crs2, 8)
                ipc.sleep(50)

            else

                com.write(dev, "dsp3/" .. string.format("%003d", OBS2), 8)
                ipc.sleep(50)

            end

        else

    	   	com.write(dev, "DMI" .. vor_name2 , 8)

        end

		com.write(dev, vor_distance2, 8)
		ipc.sleep(50)
		com.write(dev, vor_speed2, 8)
		ipc.sleep(50)

    else

        if OBS2 == 999 then

            com.write(dev, "dsp2  --", 8)
            ipc.sleep(50)
            com.write(dev, "dsp3/" .. vor_crs2, 8)
            ipc.sleep(50)

        else

            com.write(dev, "dsp2  --", 8)
            ipc.sleep(50)
            com.write(dev, "dsp3/" .. string.format("%003d", OBS2), 8)
            ipc.sleep(50)

        end

	end
		
	dme_sel = 2
	dme_open = 2	
	com_open = 0
	nav_open = 0
	
end

function Default_DME_plus (fast)

    local crs

    if RADIOS_SUBMODE == 1 then

        crs = RADIOS_DME_CRS1

    else

        crs = RADIOS_DME_CRS2

    end

    if fast then

        crs = crs + 10

    else

        crs = crs + 1

    end

    if crs > 359 then crs = crs - 360  end

    local dme = string.format("%003d", crs)

    Default_DME_set (dme)

    -- com.write(dev, "dsp0CRS\\", 8)
    -- ipc.sleep(20)
    RADIOS_MSG = true         -- need for restore "DME1" string with little pause
    RADIOS_MSG_SHORT = true   --
	ipc.set("FIP2", ipc.elapsedtime())

    if AutoDisplay then

        com.write(dev, "dsp3\\" .. dme, 8)
        ipc.sleep(20)

    end

    if RADIOS_SUBMODE == 1 then

        RADIOS_DME_CRS1 = crs

    else

        RADIOS_DME_CRS2 = crs

    end

end

function Default_DME_minus (fast)

    local crs

    if RADIOS_SUBMODE == 1 then

        crs = RADIOS_DME_CRS1

    else

        crs = RADIOS_DME_CRS2

    end

    if fast then

        crs = crs - 10

    else

        crs = crs - 1

    end

    if crs < 0 then crs = 360 + crs end

    local dme = string.format("%003d", crs)

    Default_DME_set (dme)

    -- com.write(dev, "dsp0CRS\\", 8)
    -- ipc.sleep(20)
    RADIOS_MSG = true         -- need for restore "DME1" string with little pause
    RADIOS_MSG_SHORT = true   --
	ipc.set("FIP2", ipc.elapsedtime())

    if AutoDisplay then

        com.write(dev, "dsp3\\" .. dme, 8)
        ipc.sleep(20)

    end

    if RADIOS_SUBMODE == 1 then

        RADIOS_DME_CRS1 = crs

    else

        RADIOS_DME_CRS2 = crs

    end

end


-- #######################################################
-- #######################################################

-- ## ADF ##############


function Default_ADF_select ()

    if RADIOS_MODE ~= 3 then

        RADIOS_SUBMODE = 2

    end

    RADIOS_MODE = 3 -- ADF

    if RADIOS_SUBMODE == 2 then

    	ipc.sleep(50)
        com.write(dev, "adfxxxxx", 8)

        Default_ADF_1_init (true)
        RADIOS_SUBMODE = 1

   		switch ("ADF1 Select", RADIOS, "RADIOS")

    else

    	ipc.sleep(50)
        com.write(dev, "ADFXXXXX", 8)

        Default_ADF_2_init (true)
        RADIOS_SUBMODE = 2

   		switch ("ADF2 Select", RADIOS, "RADIOS")

    end

   	ipc.sleep(50)
    com.write(dev, "RADA3", 8)
    RADIOS_CURSOR_POS = 3

end


function Default_ADF_set (s)

	freq1 = tonumber(string.sub(s, 2, 4), 16)
	freq2 = tonumber(string.sub(s, 1, 1) .. "0" .. string.sub(s, 5, 5), 16)
	
	if adf_sel == 1 then
		ipc.writeUD(0x034C, freq1)
		ipc.writeUD(0x0356, freq2)
	else
		ipc.writeUD(0x02D4, freq1)
		ipc.writeUD(0x02D6, freq2)
	end
	
end

function Default_ADF_1_init (upd)

	if dev == 0 then return end
    if upd == nil then upd = false end

	if upd or adf1_firstload then

		ipc.sleep(100)
		buffer = ipc.readUW(0x034C)
		buffer2 = ipc.readUW(0x0356)

        RADIOS_ADF_FREQ1 = string.format("%01x%03x%01x", buffer2/256, buffer, buffer2 % 0x0100)

		com.write(dev, 'adf' .. RADIOS_ADF_FREQ1, 8)
		
		adf1_firstload = false
	
	end
	
	adf_sel = 1
	dme_open = 0
	com_open = 0
	nav_open = 0
	
end

function Default_ADF_2_init (upd)

	if dev == 0 then return end
    if upd == nil then upd = false end

	if upd or adf2_firstload then

		ipc.sleep(100)
		buffer = ipc.readUW(0x02D4)
		buffer2 = ipc.readUW(0x02D6)

        RADIOS_ADF_FREQ2 = string.format("%01x%03x%01x", buffer2/256, buffer, buffer2 % 0x0100)

		com.write(dev, 'ADF' .. RADIOS_ADF_FREQ2, 8)

		adf2_firstload = false
	
	end

	adf_sel = 2
	dme_open = 0
	com_open = 0
	nav_open = 0
	
end

function Default_ADF_plus ()

    local d, freq
    local first

    if RADIOS_SUBMODE == 1 then

        freq = RADIOS_ADF_FREQ1

    else

        freq = RADIOS_ADF_FREQ2

    end

    first = tonumber(string.sub(freq, 1, 1))

    if RADIOS_CURSOR_POS == 3 then

        d = tonumber(string.sub(freq, 2, 2)) + 1

        if first == 0 and d > 9 then

            first = 1
            d = 0

        end

        if first == 1 and d > 7 then

            first = 0
            d = 1

        end

        freq = tostring(first) .. tostring(d) .. string.sub(freq, 3)

    elseif RADIOS_CURSOR_POS == 2 then

        d = tonumber(string.sub(freq, 3, 3)) + 1

        if d > 9 then d = 0 end

        freq = string.sub(freq, 1, 2) .. tostring(d) .. string.sub(freq, 4)

    elseif RADIOS_CURSOR_POS == 1 then

        d = tonumber(string.sub(freq, 4, 4)) + 1

        if d > 9 then d = 0 end

        freq = string.sub(freq, 1, 3) .. tostring(d) .. string.sub(freq, 5)

    elseif RADIOS_CURSOR_POS == 0 then

        d = tonumber(string.sub(freq, 5, 5)) + 1

        if d > 9 then d = 0 end

        freq = string.sub(freq, 1, 4) .. tostring(d)

    end

    if RADIOS_SUBMODE == 1 then

        RADIOS_ADF_FREQ1 = freq
    	ipc.sleep(50)
        com.write(dev, "adf" .. freq, 8)

    else

        RADIOS_ADF_FREQ2 = freq
    	ipc.sleep(50)
        com.write(dev, "ADF" .. freq, 8)

    end

   	ipc.sleep(50)
    com.write(dev, "RADA" .. RADIOS_CURSOR_POS, 8)

    Default_ADF_set (freq)

end


function Default_ADF_minus ()

    local d, freq
    local first

    if RADIOS_SUBMODE == 1 then

        freq = RADIOS_ADF_FREQ1

    else

        freq = RADIOS_ADF_FREQ2

    end

    first = tonumber(string.sub(freq, 1, 1))

    if RADIOS_CURSOR_POS == 3 then

        d = tonumber(string.sub(freq, 2, 2)) - 1

        if first == 0 and d < 1 then

            first = 1
            d = 7

        end

        if first == 1 and d < 0 then

            first = 0
            d = 9

        end

        freq = tostring(first) .. tostring(d) .. string.sub(freq, 3)

    elseif RADIOS_CURSOR_POS == 2 then

        d = tonumber(string.sub(freq, 3, 3)) - 1

        if d < 0 then d = 9 end

        freq = string.sub(freq, 1, 2) .. tostring(d) .. string.sub(freq, 4)

    elseif RADIOS_CURSOR_POS == 1 then

        d = tonumber(string.sub(freq, 4, 4)) - 1

        if d < 0 then d = 9 end

        freq = string.sub(freq, 1, 3) .. tostring(d) .. string.sub(freq, 5)

    elseif RADIOS_CURSOR_POS == 0 then

        d = tonumber(string.sub(freq, 5, 5)) - 1

        if d < 0 then d = 9 end

        freq = string.sub(freq, 1, 4) .. tostring(d)

    end

    if RADIOS_SUBMODE == 1 then

        RADIOS_ADF_FREQ1 = freq
    	ipc.sleep(50)
        com.write(dev, "adf" .. freq, 8)

    else

        RADIOS_ADF_FREQ2 = freq
    	ipc.sleep(50)
        com.write(dev, "ADF" .. freq, 8)

    end

   	ipc.sleep(50)
    com.write(dev, "RADA" .. RADIOS_CURSOR_POS, 8)

    Default_ADF_set (freq)

end


-- #######################################################
-- #######################################################



-- ## NAV ##############

function Default_NAV_select ()

    if RADIOS_MODE ~= 2 then

        RADIOS_SUBMODE = 2

    end

    RADIOS_MODE = 2 -- COM
    RADIOS_MhzKhz = 1 -- Mhz

	ipc.sleep(50)
    com.write(dev, "NAVi", 8)

    if RADIOS_SUBMODE == 2 then

        Default_NAV_1_init (true)
        RADIOS_SUBMODE = 1

   		switch ("NAV1 Select", RADIOS, "RADIOS")

    else

        Default_NAV_2_init (true)
        RADIOS_SUBMODE = 2

   		switch ("NAV2 Select", RADIOS, "RADIOS")

    end

	ipc.sleep(50)
	com.write(dev, 'RADMhz', 8)

end


function Default_NAV_1_set (s)

	nav1s_freq = tonumber(s,16)
	ipc.writeUD(0x311E, nav1s_freq)
	
end

function Default_NAV_2_set (s)

	nav2s_freq = tonumber(s,16)
	ipc.writeUD(0x3120, nav2s_freq)
	
end

function Default_NAV_1_swap (s)

	_logg("Swap NAV1 requested")

	ipc.control(66448)
	
	local tmp = nav1s_freq
	nav1s_freq = nav1x_freq
	nav1x_freq = tmp
	
end

function Default_NAV_2_swap (s)

	_logg("Swap NAV2 requested")

	ipc.control(66452)

	local tmp = nav2s_freq
	nav2s_freq = nav2x_freq
	nav2x_freq = tmp

end

function Default_NAV_1_init (upd)

	if dev == 0 then return end
    if upd == nil then upd = false end

	local buffer_x = ipc.readUW(0x0350)
	local buffer_s = ipc.readUW(0x311E)

	if upd or nav1_firstload or nav1s_freq ~= buffer_s or nav1x_freq ~= buffer_x then

		nav1x_freq = buffer_x
		nav1s_freq = buffer_s
		
		ipc.sleep(50)
		com.write(dev, string.format("NAVx%04x", buffer_x), 8)
		ipc.sleep(50)
		com.write(dev, string.format("NAVs%04x", buffer_s), 8)
		
		nav1_firstload = false
	
	end
	
	nav_sel = 1
	dme_open = 0
	com_open = 0
	nav_open = 1
	
end

function Default_NAV_2_init (upd)

	if dev == 0 then return end
    if upd == nil then upd = false end

    if nav2_firstload then ipc.sleep(100) end
    local buffer_x = ipc.readUW(0x0352)
    if nav2_firstload then ipc.sleep(100) end
	local buffer_s = ipc.readUW(0x3120)
	
	if buffer_s == 0 then buffer_s = 2048 end

--	_log (buffer_x .. "/" .. buffer_s)

	if upd or nav2_firstload or nav2s_freq ~= buffer_s or nav2x_freq ~= buffer_x then

		nav2x_freq = buffer_x
		nav2s_freq = buffer_s
		
		ipc.sleep(50)
		com.write(dev, string.format("NAVX%04x", buffer_x), 8)
		ipc.sleep(50)
		com.write(dev, string.format("NAVS%04x", buffer_s), 8)
		
		nav2_firstload = false
	
	end

	nav_sel = 2
	dme_open = 0
	com_open = 0
	nav_open = 2
	
end


function Default_NAVCOM_plus (nav)

    local buffer, cmd

    if nav == 0 then

        if RADIOS_SUBMODE == 1 then

            -- COM1
     	    buffer = string.format("%04x", ipc.readUW(0x311A))

            cmd = "COMs"

        else

            -- COM2
        	buffer = string.format("%04x", ipc.readUW(0x311C))

            cmd = "COMS"

        end

    else

        if RADIOS_SUBMODE == 1 then

            -- NAV1
    	    buffer = string.format("%04x", ipc.readUW(0x311E))

            cmd = "NAVs"

        else

            -- NAV2
        	buffer = string.format("%04x", ipc.readUW(0x3120))

            cmd = "NAVS"

        end

    end

    local mhz = tonumber(string.sub(buffer, 1, 2))
    local khz = tonumber(string.sub(buffer, 3, 4))

    if RADIOS_MhzKhz == 1 then

        mhz = mhz + 1

        if nav == 0 and mhz > 36 then mhz = 18 end
        if nav == 1 and mhz > 17 then mhz = 8 end

    else

        local m, mm
        m, mm = math.modf(khz / 5)

        if mm > 0 then

            khz = khz + 3

        else

            khz = khz + 2

        end

        if khz > 97 then khz = 0 end

    end

    local freq = string.format("%04d", round(mhz * 100 + khz))

    -- NAV/COM1
  	ipc.sleep(20)
    com.write(dev, cmd .. freq, 8)

    if nav == 0 then

        if RADIOS_SUBMODE == 1 then

            Default_COM_1_set (freq)

        else

            Default_COM_2_set (freq)

        end

    else

        if RADIOS_SUBMODE == 1 then

            Default_NAV_1_set (freq)

        else

            Default_NAV_2_set (freq)

        end

    end

    if RADIOS_MhzKhz == 1 then

    	ipc.sleep(20)
        com.write(dev, 'RADMhz', 8)

    else

    	ipc.sleep(20)
        com.write(dev, 'RADKhz', 8)

    end

end

function Default_NAVCOM_minus (nav)

    local buffer, cmd

    if nav == 0 then

        if RADIOS_SUBMODE == 1 then

            -- COM1
        	buffer = string.format("%04x", ipc.readUW(0x311A))

            cmd = "COMs"

        else

            -- COM2
        	buffer = string.format("%04x", ipc.readUW(0x311C))

            cmd = "COMS"

        end

    else

        if RADIOS_SUBMODE == 1 then

            -- NAV1
    	    buffer = string.format("%04x", ipc.readUW(0x311E))

            cmd = "NAVs"

        else

            -- NAV2
        	buffer = string.format("%04x", ipc.readUW(0x3120))

            cmd = "NAVS"

        end

    end


    local mhz = tonumber(string.sub(buffer, 1, 2))
    local khz = tonumber(string.sub(buffer, 3, 4))

    if RADIOS_MhzKhz == 1 then

        mhz = mhz - 1

        if nav == 0 and mhz < 18 then mhz = 36 end
        if nav == 1 and mhz < 8 then mhz = 17 end

    else

        local m, mm
        m, mm = math.modf(khz / 5)

        if mm > 0 then

            khz = khz - 2

        else

            khz = khz - 3

        end

        if khz < 0 then khz = 97 end

    end

    local freq = string.format("%04d", round(mhz * 100 + khz))

    -- NAV/COM1
  	ipc.sleep(20)
    com.write(dev, cmd .. freq, 8)

    if nav == 0 then

        if RADIOS_SUBMODE == 1 then

            Default_COM_1_set (freq)

        else

            Default_COM_2_set (freq)

        end

    else

        if RADIOS_SUBMODE == 1 then

            Default_NAV_1_set (freq)

        else

            Default_NAV_2_set (freq)

        end

    end

    if RADIOS_MhzKhz == 1 then

    	ipc.sleep(20)
        com.write(dev, 'RADMhz', 8)

    else

    	ipc.sleep(20)
        com.write(dev, 'RADKhz', 8)

    end

end



-- #######################################################
-- #######################################################

-- ## COM ##############

function Default_COM_select ()

    if RADIOS_MODE ~= 1 then

        RADIOS_SUBMODE = 2

    end

    RADIOS_MODE = 1 -- COM
    RADIOS_MhzKhz = 1 -- Mhz

	ipc.sleep(50)
    com.write(dev, "COMi", 8)

    if RADIOS_SUBMODE == 2 then

        Default_COM_1_init (true)
        RADIOS_SUBMODE = 1

   		switch ("COM1 Select", RADIOS, "RADIOS")

    else

        Default_COM_2_init (true)
        RADIOS_SUBMODE = 2

		switch ("COM2 Select", RADIOS, "RADIOS")

    end

	ipc.sleep(50)
	com.write(dev, 'RADMhz', 8)

end


function Default_COM_1_set (s)

	com1s_freq = tonumber(s,16)
	ipc.control(66371, com1s_freq)
	
end

function Default_COM_2_set (s)
	
	com2s_freq = tonumber(s,16)
	ipc.control(66443, com2s_freq)
	
end

function Default_COM_1_swap ()

	_logg("Swap COM1 requested")

	ipc.control(66372)
	
	local tmp = com1s_freq
	com1s_freq = com1x_freq
	com1x_freq = tmp

end

function Default_COM_2_swap ()

	_logg("Swap COM2 requested")

	ipc.control(66444)

	local tmp = com2s_freq
	com2s_freq = com2x_freq
	com2x_freq = tmp

end

function Default_COM_1_init (upd)

	if dev == 0 then return end
    if upd == nil then upd = false end

	local buffer_x = ipc.readUW(0x034E)
	local buffer_s = ipc.readUW(0x311A)

	if upd or com1_firstload or com1s_freq ~= buffer_s or com1x_freq ~= buffer_x then

		com1x_freq = buffer_x
		com1s_freq = buffer_s
		
		ipc.sleep(50)
		com.write(dev, string.format("COMx%04x", buffer_x), 8)
		ipc.sleep(50)
		com.write(dev, string.format("COMs%04x", buffer_s), 8)
		
		com1_firstload = false
	
	end
	
	com_sel = 1
	dme_open = 0
	com_open = 1
	nav_open = 0

end

function Default_COM_2_init (upd)

	if dev == 0 then return end
    if upd == nil then upd = false end

	local buffer_x = ipc.readUW(0x3118)
	local buffer_s = ipc.readUW(0x311C)
	
	if upd or com2_firstload or com2s_freq ~= buffer_s or com2x_freq ~= buffer_x then

		com2x_freq = buffer_x
		com2s_freq = buffer_s

		ipc.sleep(50)
		com.write(dev, string.format("COMX%04x", buffer_x), 8)
		ipc.sleep(50)
		com.write(dev, string.format("COMS%04x", buffer_s), 8)
		
		com2_firstload = false
	
	end
	
	com_sel = 2
	dme_open = 0
	com_open = 2
	nav_open = 0
	
end


-- #######################################################
-- #######################################################



-- ## Transponder ##############

function Default_XPND_select ()

    RADIOS_MODE = 5 -- XPND

   	ipc.sleep(50)
    com.write(dev, "TRNXXXXX", 8)

    Default_XPND_init (true)

   	ipc.sleep(50)
    com.write(dev, "RADT3", 8)
    RADIOS_CURSOR_POS = 3

	switch ("XPND Select", RADIOS, "RADIOS")

end

function Default_XPND_set (s)

	ipc.control(65715, tonumber(s,16))

end

function Default_XPND_init (upd)
	
	if dev == 0 then return end
    if upd == nil then upd = false end

	if upd or trn_firstload then

		ipc.sleep(50)
		local buffer = ipc.readUW(0x0354)

        RADIOS_XPND_CODE = string.format("%04x", buffer)

		com.write(dev, "TRN" .. RADIOS_XPND_CODE, 8)
		ipc.sleep(50)

		trn_firstload = false

	end
	
	dme_open = 0
	com_open = 0
	nav_open = 0
	
end

function Default_XPND_plus ()

    local d, code

    code = RADIOS_XPND_CODE

    if RADIOS_CURSOR_POS == 3 then

        d = tonumber(string.sub(code, 1, 1)) + 1

        if d > 7 then d = 0 end

        code = tostring(d) .. string.sub(code, 2)

    elseif RADIOS_CURSOR_POS == 2 then

        d = tonumber(string.sub(code, 2, 2)) + 1

        if d > 7 then d = 0 end

        code = string.sub(code, 1, 1) .. tostring(d) .. string.sub(code, 3)

    elseif RADIOS_CURSOR_POS == 1 then

        d = tonumber(string.sub(code, 3, 3)) + 1

        if d > 7 then d = 0 end

        code = string.sub(code, 1, 2) .. tostring(d) .. string.sub(code, 4)

    elseif RADIOS_CURSOR_POS == 0 then

        d = tonumber(string.sub(code, 4, 4)) + 1

        if d > 7 then d = 0 end

        code = string.sub(code, 1, 3) .. tostring(d)

    end

    RADIOS_XPND_CODE = code
 	ipc.sleep(50)
    com.write(dev, "TRN" .. code, 8)

   	ipc.sleep(50)
    com.write(dev, "RADT" .. RADIOS_CURSOR_POS, 8)

end

function Default_XPND_minus ()

    local d, code

    code = RADIOS_XPND_CODE

    if RADIOS_CURSOR_POS == 3 then

        d = tonumber(string.sub(code, 1, 1)) - 1

        if d < 0 then d = 7 end

        code = tostring(d) .. string.sub(code, 2)

    elseif RADIOS_CURSOR_POS == 2 then

        d = tonumber(string.sub(code, 2, 2)) - 1

        if d < 0 then d = 7 end

        code = string.sub(code, 1, 1) .. tostring(d) .. string.sub(code, 3)

    elseif RADIOS_CURSOR_POS == 1 then

        d = tonumber(string.sub(code, 3, 3)) - 1

        if d < 0 then d = 7 end

        code = string.sub(code, 1, 2) .. tostring(d) .. string.sub(code, 4)

    elseif RADIOS_CURSOR_POS == 0 then

        d = tonumber(string.sub(code, 4, 4)) - 1

        if d < 0 then d = 7 end

        code = string.sub(code, 1, 3) .. tostring(d)

    end

    RADIOS_XPND_CODE = code
 	ipc.sleep(50)
    com.write(dev, "TRN" .. code, 8)

   	ipc.sleep(50)
    com.write(dev, "RADT" .. RADIOS_CURSOR_POS, 8)

end


-- #######################################################
-- #######################################################


-- ## Knob mode toggle ###############

-- CURRENT KNOB MODE TOGGLE
function KNOB_MODE_toggle (skip, group)

	-- _log("KNOB MODE toggle: " .. group)

	if group == "MINS" then
	
		if MINSmode == "A" then
		
			MINSmode = "B"
			
		else
		
			MINSmode = "A"
		
		end
		
		if ipc.get("EFISmode") == 1 then 
		
			-- updating display
			switch (MINSmode .. " SHOW", MINS1, "MINS")

		elseif ipc.get("EFISmode") == 2 then 
		
			-- updating display
			switch (MINSmode .. " SHOW", MINS2, "MINS")

		else
		
			-- updating display
			switch (MINSmode .. " SHOW", MINS3, "MINS")

		end
		
		_log("KNOB toggle :: MINS " .. MINSmode)
	
		return
	
	end

	
	if group == "BARO" then
	
		if BAROmode == "A" then
		
			BAROmode = "B"
			
		else
		
			BAROmode = "A"
		
		end
		
		if ipc.get("EFISmode") == 1 then 
		
			-- updating display
			switch (BAROmode .. " SHOW", BARO1, "BARO")

		elseif ipc.get("EFISmode") == 2 then 
		
			-- updating display
			switch (BAROmode .. " SHOW", BARO2, "BARO")

		else
		
			-- updating display
			switch (BAROmode .. " SHOW", BARO3, "BARO")

		end
		
		_log("KNOB toggle :: BARO " .. BAROmode)
	
		return
	
	end

	
	if group == "CTR" then
	
		if CTRmode == "A" then
		
			CTRmode = "B"
			
		else
		
			CTRmode = "A"
		
		end
		
		if ipc.get("EFISmode") == 1 then 
		
			-- updating display
			switch (CTRmode .. " SHOW", CTR1, "CTR")

		elseif ipc.get("EFISmode") == 2 then 
		
			-- updating display
			switch (CTRmode .. " SHOW", CTR2, "CTR")

		else
		
			-- updating display
			switch (CTRmode .. " SHOW", CTR3, "CTR")

		end
		
		_log("KNOB toggle :: CTR " .. CTRmode)
	
		return
	
	end


	if group == "TFC" then
	
		if TFCmode == "A" then
		
			TFCmode = "B"
			
		else
		
			TFCmode = "A"
		
		end
		
		if ipc.get("EFISmode") == 1 then 
		
			-- updating display
			switch (TFCmode .. " SHOW", TFC1, "TFC")

		elseif ipc.get("EFISmode") == 2 then 
		
			-- updating display
			switch (TFCmode .. " SHOW", TFC2, "TFC")

		else
		
			-- updating display
			switch (TFCmode .. " SHOW", TFC3, "TFC")

		end
		
		_log("KNOB toggle :: TFC " .. TFCmode)
	
		return
	
	end


------------------------------------------------------------------------	

	if group == "CRS" then
	
		if CRSmode == "A" then
		
			CRSmode = "B"
			
		else
		
			CRSmode = "A"
		
		end
		
		if ipc.get("MCPmode") == 1 then 
		
			-- updating display
			switch (CRSmode .. " SHOW", CRS1, "CRS")

		elseif ipc.get("MCPmode") == 2 then 
		
			-- updating display
			switch (CRSmode .. " SHOW", CRS2, "CRS")

		else
		
			-- updating display
			switch (CRSmode .. " SHOW", CRS3, "CRS")

		end
		
		_log("KNOB toggle :: CRS " .. CRSmode)
	
		return
	
	end


	if group == "SPD" then
	
		if SPDmode == "A" then
		
			SPDmode = "B"
			
		else
		
			SPDmode = "A"
		
		end
		
		if ipc.get("MCPmode") == 1 then 
		
			-- updating display
			switch (SPDmode .. " SHOW", SPD1, "SPD")

		elseif ipc.get("MCPmode") == 2 then 
		
			-- updating display
			switch (SPDmode .. " SHOW", SPD2, "SPD")

		else
		
			-- updating display
			switch (SPDmode .. " SHOW", SPD3, "SPD")

		end
		
		_log("KNOB toggle :: SPD " .. SPDmode)
	
		return
	
	end


	if group == "HDG" then
	
		if HDGmode == "A" then
		
			HDGmode = "B"
			
		else
		
			HDGmode = "A"
		
		end
		
		if ipc.get("MCPmode") == 1 then 
		
			-- updating display
			switch (HDGmode .. " SHOW", HDG1, "HDG")

		elseif ipc.get("MCPmode") == 2 then 
		
			-- updating display
			switch (HDGmode .. " SHOW", HDG2, "HDG")

		else
		
			-- updating display
			switch (HDGmode .. " SHOW", HDG3, "HDG")

		end
		
		_log("KNOB toggle :: HDG " .. HDGmode)
	
		return
	
	end


	if group == "ALT" then
	
		if ALTmode == "A" then
		
			ALTmode = "B"
			
		else
		
			ALTmode = "A"
		
		end
		
		if ipc.get("MCPmode") == 1 then 
		
			-- updating display
			switch (ALTmode .. " SHOW", ALT1, "ALT")

		elseif ipc.get("MCPmode") == 2 then
		
			-- updating display
			switch (ALTmode .. " SHOW", ALT2, "ALT")

		else
		
			-- updating display
			switch (ALTmode .. " SHOW", ALT3, "ALT")

		end
		
		_log("KNOB toggle :: ALT " .. ALTmode)
	
		return
	
	end


	if group == "VVS" then
	
		if VVSmode == "A" then
		
			VVSmode = "B"
			
		else
		
			VVSmode = "A"
		
		end
		
		if ipc.get("MCPmode") == 1 then 
		
			-- updating display
			switch (VVSmode .. " SHOW", VVS1, "VVS")

		elseif ipc.get("MCPmode") == 2 then 
		
			-- updating display
			switch (VVSmode .. " SHOW", VVS2, "VVS")

		else
		
			-- updating display
			switch (VVSmode .. " SHOW", VVS3, "VVS")

		end
		
		_log("KNOB toggle :: VVS " .. VVSmode)
	
		return
	
	end

	
end

-- #######################################################
-- #######################################################



-- ## Block modes ##############

function EFIS_MODE_toggle ()
	
	local em = ipc.get("EFISmode") + 1
	
	_log("EFIS mode toggle: " .. ipc.get("EFISmode"))
	
	if EFIS3["ENABLED"] then
	
		-- 3 modes enabled
		if em > 3 then em = 1 end
		
	else
	
		-- 2 modes only
		if em > 2 then em = 1 end
		
	end
	
	if em == 1 then
	
		EFIS_MODE_one ()
		
	elseif em == 2 then
	
		EFIS_MODE_two ()
		
	else

		EFIS_MODE_three ()
	
	end
	
end

function EFIS_MODE_one ()
	
	ipc.set("EFISmode", 1)
	
	local ident = "mod1"

	if type (EFIS1["IDENT"]) == "string" and string.len(EFIS1["IDENT"]) > 0 
	then ident = string.sub(EFIS1["IDENT"], 1, 4) end
	
	DspShow("EFIS", ident, true, true)
	-- ipc.sleep(500)
	
end

function EFIS_MODE_two ()
	
	ipc.set("EFISmode", 2)

	local ident = "mod2"

	if type (EFIS2["IDENT"]) == "string" and string.len(EFIS2["IDENT"]) > 0 
	then ident = string.sub(EFIS2["IDENT"], 1, 4) end
	
	DspShow("EFIS", ident, true, true)
	-- ipc.sleep(500)

    ipc.set("EFISalt", ipc.elapsedtime())
	
end

function EFIS_MODE_three ()
	
	ipc.set("EFISmode", 3)

	local ident = "mod3"

	if type (EFIS3["IDENT"]) == "string" and string.len(EFIS3["IDENT"]) > 0 
	then ident = string.sub(EFIS3["IDENT"], 1, 4)  end
	
	DspShow("EFIS", ident, true, true)
	-- ipc.sleep(500)
	
    ipc.set("EFISalt", ipc.elapsedtime())

end

-- #######################################################
-- #######################################################



-- ### MCP MODE ##############

function MCP_MODE_toggle ()
	
	local mm = ipc.get("MCPmode") + 1
	
	if MCP3["ENABLED"] then
	
		-- 3 modes enabled
		if mm > 3 then mm = 1 end
		
	else
	
		-- 2 modes only
		if mm > 2 then mm = 1 end
		
	end
	
	_log("MCP mode toggle: " .. ipc.get("MCPmode") .. " -> " .. mm)

	if mm == 1 then
	
		MCP_MODE_one ()
		
	elseif mm == 2 then
	
		MCP_MODE_two ()
		
	else

		MCP_MODE_three ()
	
	end
	
end

function MCP_MODE_one ()
	
	InitDsp()
	
    if not ipc.get("APlock") == 1 then

    	ipc.set("DSPmode", 1)

    end

	ipc.set("MCPmode", 1)
	
	local ident = "mod1"

	if type (MCP1["IDENT"]) == "string" and string.len(MCP1["IDENT"]) > 0 
	then ident = string.sub(MCP1["IDENT"], 1, 4) end
	
	DspShow("MCP", ident, true)
	ipc.sleep(500)
	
end

function MCP_MODE_two ()
	
	ipc.set("MCPmode", 2)

	local ident = "mod2"

	if type (MCP2["IDENT"]) == "string" and string.len(MCP2["IDENT"]) > 0 
	then ident = string.sub(MCP2["IDENT"], 1, 4) end
	
	DspShow("MCP", ident, true)
	ipc.sleep(1000)
	
	if _MCP1 () then DSP_MODE_two () end
	
    ipc.set("MCPalt", ipc.elapsedtime())

end

function MCP_MODE_three ()
	
	ipc.set("MCPmode", 3)

	local ident = "mod3"

	if type (MCP3["IDENT"]) == "string" and string.len(MCP3["IDENT"]) > 0 
	then ident = string.sub(MCP3["IDENT"], 1, 4)  end
	
	DspShow("MCP", ident, true)
	ipc.sleep(1000)
	
	if _MCP1 () then DSP_MODE_two () end

    ipc.set("MCPalt", ipc.elapsedtime())

end

-- #######################################################
-- #######################################################



-- ### MCP MODE ##############

function USER_MODE_toggle ()

	local um = ipc.get("USERmode") + 1
	
	_log("User mode toggle: " .. ipc.get("USERmode"))
	
	if USER3["ENABLED"] then
	
		-- 3 modes enabled
		if um > 3 then um = 1 end
		
	else
	
		-- 2 modes only
		if um > 2 then um = 1 end
		
	end
	
	if um == 1 then
	
		USER_MODE_one ()
		
	elseif um == 2 then
	
		USER_MODE_two ()
		
	else

		USER_MODE_three ()
	
	end
	
end

function USER_MODE_one ()
	
	ipc.set("USERmode", 1)
	
	local ident = "mod1"

	if type (USER1["IDENT"]) == "string" and string.len(USER1["IDENT"]) > 0 
	then ident = string.sub(USER1["IDENT"], 1, 4) end
	
	DspShow("USER", ident, true, true)
	-- ipc.sleep(500)
	
end

function USER_MODE_two ()
	
	ipc.set("USERmode", 2)

	local ident = "mod2"

	if type (USER2["IDENT"]) == "string" and string.len(USER2["IDENT"]) > 0 
	then ident = string.sub(USER2["IDENT"], 1, 4) end
	
	DspShow("USER", ident, true, true)
	-- ipc.sleep(500)
	
    ipc.set("USERalt", ipc.elapsedtime())

end

function USER_MODE_three ()
	
	ipc.set("USERmode", 3)

	local ident = "mod3"

	if type (USER3["IDENT"]) == "string" and string.len(USER3["IDENT"]) > 0 
	then ident = string.sub(USER3["IDENT"], 1, 4)  end
	
	DspShow("USER", ident, true, true)
	-- ipc.sleep(500)
	
    ipc.set("USERalt", ipc.elapsedtime())

end

-- #######################################################
-- #######################################################



-- ## DISPLAY MODEs ##############

function DSP_MODE_toggle ()

	if ipc.get("DSPmode") == 1 then
	
		DSP_MODE_two ()
		
	else
	
		DSP_MODE_one ()
		
	end
	
end

function DSP_MODE_one ()
	
	if ipc.get("APlock") == 1 then return end
	
	ipc.set("DSPmode", 1)

	InitDsp()

	_log ("DSP MODE toggle: " .. ipc.get("DSPmode"))

    DspClearMed ()

	DspShow(" AP ", "MODE")


end

function DSP_MODE_two ()
	
	ipc.set("DSPmode", 2)
	DisplayFlightInfo ()
	_log ("DSP MODE toggle: " .. ipc.get("DSPmode"))
	
end

-- #######################################################
-- #######################################################



-- ## Flight info ############
function DisplayFlightInfo ()

    local info = "M" .. tostring(ipc.get("EFISmode")) .. tostring(ipc.get("MCPmode")) .. tostring(ipc.get("USERmode"))

    if info == "M111" then

        if FLIGHT_INFO1 ~= '' then

            DspMed1 (FLIGHT_INFO1, false)

        else

            DspMed1 (FLIGHT_INFO_TEXT, false)

        end

    else

        Dsp0(info)

    end

	mcp_tmp = 0

    if FLIGHT_INFO2 ~= '' then

        if info == "M111" then

            DspMed2 (FLIGHT_INFO2, false)

        end

    else

		if ipc.readUD(0x07BC) ~= AP_STATE then

            AP_STATE = ipc.readUD(0x07BC)

            if AP_STATE == 1 then

                Dsp1("*AP*")

            else

			    Dsp1("-ap-")

            end

        end

    end

	---[[ Manual magnetic deviation counting
	
	local cur_hdg = ipc.readUD(0x0580)*360/(65536*65536)
	local cur_hdg_var = ipc.readUW(0x02A0)*360/65536

	if cur_hdg_var < 180 then 
		cur_hdg = round(cur_hdg - cur_hdg_var)
	else
		cur_hdg = round(cur_hdg + (360 - cur_hdg_var))
	end

	if cur_hdg < 0 then cur_hdg = 360 - cur_hdg end
	if cur_hdg > 360 then cur_hdg = cur_hdg - 360 end


	--]]
	
	-- easier but gyro drift effected
	-- cur_hdg = ipc.readDBL(0x2B00)
	
	local cur_vs = ipc.readUW(0x0842) -- *3.28084

    if _MCP1 () then

    	if cur_vs > 32766 then
	   	   cur_vs = 65536 - cur_vs
    		cur_vs = round(cur_vs * 3.28084 / 100)
    		Dsp1(string.format("v+%02d", cur_vs))
    	else
    	   	cur_vs = round(cur_vs * 3.28084 / 100)
    		Dsp1(string.format("v-%02d", cur_vs))
        end

    else

    	if cur_vs > 32766 then
	   	   cur_vs = 65536 - cur_vs
    		cur_vs = round(cur_vs * 3.28084 / 100)
    	else
    	   	cur_vs = round(cur_vs * 3.28084 / 100) * (-1)
        end

        DspVVS(cur_vs)

    end
	
	local cur_spd = round(ipc.readUD(0x02BC) / 128)
	
	local cur_alt = round(ipc.readUD(0x3324) / 100)
	if cur_alt > 21474835 then
		cur_alt = 0
	end
	
	DspSPD(cur_spd)
	DspHDG(cur_hdg)
	DspALT(cur_alt)
	
    if _MCP2 () then

       	ipc.sleep(20)
    	com.write(dev, "DSPE00 \\", 8)

    end

end

function DspLocked()

	if ipc.get("DSPmode") == 2 then
	
		Dsp0("****")
		ipc.sleep(250)
		Dsp0("INFO")
	
		_logg ("DSP Lock check - Locked!")

		return true
		
	else
	
		-- _logg ("DSP Lock check - Not locked...")
		return false
	
	end
	
end

function AutopilotDisplayBlocked ()

	ipc.set("APlock", 1)
	DSP_MODE_two ()
	
end

-- #######################################################
-- #######################################################


-- ## INIT ###############
-- connect to MCP OM-port
function ConnectMCP ()

	-- Connect to VRi MCP
	
	-- COM port params
	speed = 115200 
	handshake = 0  

    -- Trying to make a persistent connection to VRI MCP
    dev = com.open(ipc.get("VRIdevice"), speed, handshake)


	if dev == 0 then
	
		_err("Could not open MCP Combo port!")
		
		ipc.display("VRi MCP Combo is ENABLED but not found on port " .. string.upper(ipc.get("VRIdevice")) , 10)
		
		-- ipc.sleep(5000)
	end

end

function InitMCP ()

	if dev == 0 then return end

	-- initialize MCP Combo
	ipc.sleep(150)
	com.write(dev, "CMDRST", 8)
	ipc.sleep(150)
    com.write(dev, "CMDCON", 8)
    ipc.sleep(150)

    if _MCP2 () then

        -- radios display
        ipc.sleep(1100)
        DspRadioLong1("   Powered by   ")
        DspRadioLong2("   LINDA v1.1   ")

        ipc.sleep(100)
        com.write(dev, "DSPI", 8)
        ipc.sleep(100)

        DspLong1("LINDA init...")
        DspLong2(ipc.get("acft_handle"))

        ipc.sleep(1000)
        com.write(dev, "DSPI", 8)
        ipc.sleep(150)
        com.write(dev, "F/DOFF", 8)
        ipc.sleep(50)
        com.write(dev, "A/TOFF", 8)
        ipc.sleep(50)
        com.write(dev, 'DSP1\\\\\\ ', 8)
        ipc.sleep(50)
        com.write(dev, 'DSP3 \\\\ ', 8)
        ipc.sleep(50)
        com.write(dev, 'DSP9\\\\\\ ', 8)
        ipc.sleep(50)
        com.write(dev, 'DSPB \\\\ ', 8)
        -- ipc.sleep(50)
        -- com.write(dev, 'HDG000', 8)

        -- moving indications
        ipc.sleep(50)
        com.write(dev, 'DSP4HDG ', 8)
        ipc.sleep(50)
        com.write(dev, 'DSPC--- ', 8)

        ipc.sleep(50)
        com.write(dev, 'SPD000', 8)

    else

     	com.write(dev, "CMDFR", 8)
	    ipc.sleep(50)

    end

end

function Init ()

	-- Init common vars
	-- CommonInitVars ()

	-- fallback init vars
	if type(FallbackInitVars) == "function" then FallbackInitVars () end
		
	-- Init current module vars if defined in module
	if type(InitVars) == "function" then InitVars() end

	if ipc.get("VRI_ENABLED") == 1 then

		-- COM-port connection
		ConnectMCP()
		
		if dev ~= 0 then
		
			-- Initialize MCP
			InitMCP()
			
			-- Init display
			ipc.sleep(1000)

			InitDsp()
			
			if ipc.get("VRItype") == "mcp2" then

                DspMed1 (" Ready!")
                --DspClearMed()

            else

                DspShow("Rdy!", "    ")

            end
			
		end
		
	end
	
	init = true

end


-- ## Display functions ###############

function DspMed1 (s, clear)

	if dev == 0 then return end
    if s == DSP_MSG_PREV1 then return end
    DSP_MSG_PREV1 = s

    if not _MCP2 () then

        Dsp0(s)
        return

    end

    if clear == nil then clear = true end

    s = DspStrMed (s)

    ipc.sleep(20)
    com.write(dev, "DSP1" .. "\\\\\\" .. string.sub(s, 1, 1), 8)
    ipc.sleep(20)
    com.write(dev, "DSP2" .. string.sub(s, 2, 5), 8)
    ipc.sleep(20)
    com.write(dev, "DSP3" .. string.sub(s, 6, 8) .. "\\", 8)
    ipc.sleep(20)

	ipc.set("FIP", ipc.elapsedtime())

    DSP_MSG1 = clear

end

function DspMed2 (s, clear)

	if dev == 0 then return end
    if s == DSP_MSG_PREV1 then return end
    DSP_MSG_PREV2 = s

    if not _MCP2 () then

        Dsp1(s)
        return

    end

    if clear == nil then clear = true end

    s = DspStrMed (s)

    ipc.sleep(20)
    com.write(dev, "DSP9" .. "\\\\\\" .. string.sub(s, 1, 1), 8)
    ipc.sleep(20)
    com.write(dev, "DSPA" .. string.sub(s, 2, 5), 8)
    ipc.sleep(20)
    com.write(dev, "DSPB" .. string.sub(s, 6, 8) .. "\\", 8)
    ipc.sleep(20)

	ipc.set("FIP", ipc.elapsedtime())

    DSP_MSG2 = clear

end


function DspRadiosShortClear ()

	if dev == 0 then return end

    if not _MCP2 () then return end

    ipc.sleep(20)
    com.write(dev, "dsp3    ", 8)

end

function DspRadiosMedClear ()

	if dev == 0 then return end

    if not _MCP2 () then return end

    ipc.sleep(20)
    com.write(dev, "dsp1    ", 8)
    ipc.sleep(20)
    com.write(dev, "dsp2    ", 8)
    ipc.sleep(20)
    com.write(dev, "dsp3    ", 8)

end


function DspRadioShort (s)

	if dev == 0 then return end

    if not _MCP2 () then return end

	while string.len(s) < 4 do s = " " .. s end

    ipc.sleep(20)
    com.write(dev, "dsp3" .. string.sub(s, 1, 4), 8)

	ipc.set("FIP2", ipc.elapsedtime())

    RADIOS_MSG = true
    RADIOS_MSG_SHORT = true

end

function DspRadioMed (s)

	if dev == 0 then return end

    if not _MCP2 () then return end

	while string.len(s) < 12 do s = " " .. s end

    ipc.sleep(20)
    com.write(dev, "dsp1" .. string.sub(s, 1, 4), 8)
    ipc.sleep(20)
    com.write(dev, "dsp2" .. string.sub(s, 5, 8), 8)
    ipc.sleep(20)
    com.write(dev, "dsp3" .. string.sub(s, 9, 12), 8)
    ipc.sleep(20)

	ipc.set("FIP2", ipc.elapsedtime())
    RADIOS_MSG = true

end

function DspRadioLong1 (s)

	if dev == 0 then return end

    if not _MCP2 () then return end

    s = DspStrLong (s)

    ipc.sleep(20)
    com.write(dev, "dsp0" .. string.sub(s, 1, 4), 8)
    ipc.sleep(20)
    com.write(dev, "dsp1" .. string.sub(s, 5, 8), 8)
    ipc.sleep(20)
    com.write(dev, "dsp2" .. string.sub(s, 9, 12), 8)
    ipc.sleep(20)
    com.write(dev, "dsp3" .. string.sub(s, 13, 16), 8)
    ipc.sleep(20)

	ipc.set("FIP2", ipc.elapsedtime())

end

function DspRadioLong2 (s)

	if dev == 0 then return end

    if not _MCP2 () then return end

    s = DspStrLong (s)

    ipc.sleep(20)
    com.write(dev, "dsp4" .. string.sub(s, 1, 4), 8)
    ipc.sleep(20)
    com.write(dev, "dsp5" .. string.sub(s, 5, 8), 8)
    ipc.sleep(20)
    com.write(dev, "dsp6" .. string.sub(s, 9, 12), 8)
    ipc.sleep(20)
    com.write(dev, "dsp7" .. string.sub(s, 13, 16), 8)
    ipc.sleep(20)

	ipc.set("FIP2", ipc.elapsedtime())

end

function DspLong1 (s)

	if dev == 0 then return end

    if not _MCP2 () then return end

    s = DspStrLong (s)

    ipc.sleep(20)
    com.write(dev, "DSP0" .. string.sub(s, 1, 4), 8)
    ipc.sleep(20)
    com.write(dev, "DSP1" .. string.sub(s, 5, 8), 8)
    ipc.sleep(20)
    com.write(dev, "DSP2" .. string.sub(s, 9, 12), 8)
    ipc.sleep(20)
    com.write(dev, "DSP3" .. string.sub(s, 13, 16), 8)
    ipc.sleep(20)

	ipc.set("FIP", ipc.elapsedtime())

end

function DspLong2 (s)

	if dev == 0 then return end

    if not _MCP2 () then return end

    s = DspStrLong (s)

    ipc.sleep(20)
    com.write(dev, "DSP8" .. string.sub(s, 1, 4), 8)
    ipc.sleep(20)
    com.write(dev, "DSP9" .. string.sub(s, 5, 8), 8)
    ipc.sleep(20)
    com.write(dev, "DSPA" .. string.sub(s, 9, 12), 8)
    ipc.sleep(20)
    com.write(dev, "DSPB" .. string.sub(s, 13, 16), 8)
    ipc.sleep(20)

	ipc.set("FIP", ipc.elapsedtime())

end


-- MCP Display operations
function DspClear ()

	if dev == 0 then return end

    local cmd1, cmd2

    if ipc.get("VRItype") == "mcp" then

        cmd1 = "DSP0"
        cmd2 = "DSP1"

    else

        cmd1 = "MSG0"
        cmd2 = "MSG1"

    end

    ipc.sleep(20)
	com.write(dev, cmd1 .. "    ", 8)
	ipc.sleep(20)
	com.write(dev, cmd2 .. "    ", 8)
end


function DspClearMed ()

	if dev == 0 then return end

    if not _MCP2 () then return end

    local cmd1, cmd2

    ipc.sleep(20)
	com.write(dev, "DSP1\\\\\\ ", 8)
	ipc.sleep(20)
	com.write(dev, "DSP2    ", 8)
	ipc.sleep(20)
	com.write(dev, "DSP3   \\", 8)
	ipc.sleep(20)
	com.write(dev, "DSP9\\\\\\ ", 8)
	ipc.sleep(20)
	com.write(dev, "DSPA    ", 8)
	ipc.sleep(20)
	com.write(dev, "DSPB   \\", 8)
	ipc.sleep(20)

end

function DspClearMed1 ()

	if dev == 0 then return end

    if not _MCP2 () then return end

    ipc.sleep(20)
	com.write(dev, "DSP1\\\\\\ ", 8)
	ipc.sleep(20)
	com.write(dev, "DSP2    ", 8)
	ipc.sleep(20)
	com.write(dev, "DSP3   \\", 8)
	ipc.sleep(20)

end

function DspClearMed2 ()

	if dev == 0 then return end

    if not _MCP2 () then return end

    local cmd1, cmd2

    ipc.sleep(20)
	com.write(dev, "DSP9\\\\\\ ", 8)
	ipc.sleep(20)
	com.write(dev, "DSPA    ", 8)
	ipc.sleep(20)
	com.write(dev, "DSPB   \\", 8)
	ipc.sleep(20)

end

function DspStr (s)

	s = tostring(s)

	while string.len(s) < 4 do s = s .. " " end
	
	return s

end 

function DspStrLong (s)

	s = tostring(s)

	while string.len(s) < 24 do s = s .. " " end

	return s

end

function DspStrMed (s)

	s = tostring(s)

	while string.len(s) < 8 do s = s .. " " end

	return s

end

function DspNum (i, digits)

	if digits == 2 then
	
		s = string.format("%02d", round(tonumber(i)))
	
	else
	
		s = string.format("%03d", round(tonumber(i)))
		
	end
	
	-- while string.len(s) < 3 do s = '0' .. s end

    if i == 0 then s = "000" end
	
	return s

end 

function DspShow (line1, line2, clear, prev)

	if dev == 0 then return end

	if clear == nil then clear = true end
	if prev == nil then prev = false end

    if DSP_MSG_PREV1 == line1 and DSP_MSG_PREV2 == line2 then return end
    DSP_MSG_PREV1 = line1
    DSP_MSG_PREV2 = line2

	line1 = DspStr(line1)
	line2 = DspStr(line2)
	
    local cmd1, cmd2

    if ipc.get("VRItype") == "mcp" then

        cmd1 = "DSP0"
        cmd2 = "DSP1"

    else

        cmd1 = "MSG0"
        cmd2 = "MSG1"

        if DSP_MSG or true then

            ipc.sleep(20)
    	    com.write(dev, 'DSP1\\\\\\ ', 8)
            ipc.sleep(20)
    	    com.write(dev, 'DSP3    ', 8)
            ipc.sleep(20)
    	    com.write(dev, 'DSP9\\\\\\ ', 8)
            ipc.sleep(20)
    	    com.write(dev, 'DSPB    ', 8)

        end

    end

    ipc.sleep(20)
	com.write(dev, cmd1 .. line1, 8)
	ipc.sleep(20)
	com.write(dev, cmd2 .. line2, 8)


	if clear then 
	
		dsp_count = ipc.elapsedtime() -- start counting
		ipc.set("FIP", ipc.elapsedtime())
		
		-- clear prev DSP state
		if not prev then
			
			dsp0_prev = ""
			dsp1_prev = ""
		
		end
	
	else

		dsp0_prev = line1 -- save last string sent to DSP
		dsp1_prev = line2 -- save last string sent to DSP
	
	end
	
    DSP_MSG = true

end

function Dsp0 (s)

	if dev == 0 then return end
	if DSP_MSG_PREV1 == s then return end
	DSP_MSG_PREV1 = s

	s = DspStr(s)
	
    local cmd

    if _MCP1 () then

        cmd = "DSP0"

    else

        cmd = "MSG0"

    end

	ipc.sleep(20)
	com.write(dev, cmd .. s, 8)

	dsp0_prev = s -- save last string sent to DSP

end

function Dsp1 (s)

	if dev == 0 then return end
	if DSP_MSG_PREV2 == s then return end
    DSP_MSG_PREV2 = s

	s = DspStr(s)

    local cmd

    if _MCP1 () then

        cmd = "DSP1"

    else

        cmd = "MSG1"

    end

	ipc.sleep(20)
	com.write(dev, cmd .. s, 8)
	
	dsp1_prev = s -- save last string sent to DSP

end

function DspSPD (i)

	if dev == 0 then return end
	
	ipc.sleep(20)
	com.write(dev, "SPD" .. DspNum(i), 8)
	
end

function DspSPDs (s)

	if dev == 0 then return end

	ipc.sleep(20)
	com.write(dev, "DSP8\\\\\\" .. string.sub(s, 1, 1), 8)
	ipc.sleep(20)
	com.write(dev, "DSP9" .. string.sub(s, 2, 4) .. ' \\', 8)

end


function DspHDG (i)

	if dev == 0 then return end

    if _MCP1() then

    	ipc.sleep(20)
	    com.write(dev, "HDG" .. DspNum(i), 8)

    else

    	ipc.sleep(20)
	    com.write(dev, "DSPC" .. DspNum(i) .. '\\', 8)

    end

end

function DspHDGs (s)

	if dev == 0 then return end

   	ipc.sleep(20)
    com.write(dev, "DSPC" .. s, 8)

end


function DspALT (i)

	if dev == 0 then return end

	ipc.sleep(20)
	com.write(dev, "ALT" .. DspNum(i), 8)

end

function DspALTs (s)

	if dev == 0 then return end

	ipc.sleep(20)
	com.write(dev, "DSPD  " .. string.sub(s, 1, 2), 8)
	ipc.sleep(20)
	com.write(dev, "DSPE" .. string.sub(s, 3, 4) .. '  ', 8)

end

function DspVVS (i)

	if dev == 0 then return end

    local line = '000'

    i = i * 10

    if i > 0 then

        line = string.format("+%003d", i)

    elseif i == 0 then

        line = ' ' .. line

    elseif i < 0 then

        line = string.format("-%003d", math.abs(i))

    end

    -- _log('L: ' .. line)

	ipc.sleep(20)
	com.write(dev, "VVS" .. line, 8)

end


function DspVVSs (s)

	if dev == 0 then return end

   	ipc.sleep(20)
    com.write(dev, "DSPF" .. s, 8)

end


function DspFD (i)

	if dev == 0 then return end

    if i == 1 then

    	ipc.sleep(20)
	    com.write(dev, "F/DON", 8)

    else

    	ipc.sleep(20)
	    com.write(dev, "F/DOFF", 8)

    end

end


function DspAT (i)

	if dev == 0 then return end

    if i == 1 then

    	ipc.sleep(20)
	    com.write(dev, "A/TON", 8)

    else

    	ipc.sleep(20)
	    com.write(dev, "A/TOFF", 8)

    end

end


function DspSPD_AP_on ()

	if dev == 0 then return end
    if _MCP1 () then return end

   	ipc.sleep(20)
    com.write(dev, "DSP1\\\\*\\", 8)

end

function DspSPD_AP_off ()

	if dev == 0 then return end
    if _MCP1 () then return end

   	ipc.sleep(20)
    com.write(dev, "DSP1\\\\ \\", 8)

end

function DspSPD_N1_on ()

	if dev == 0 then return end
    if _MCP1 () then return end

   	ipc.sleep(20)
    com.write(dev, "DSP0\\\\\\ ", 8)
   	ipc.sleep(20)
    com.write(dev, "DSP1N1*\\", 8)

end

function DspSPD_N1_off ()

	if dev == 0 then return end
    if _MCP1 () then return end

   	ipc.sleep(20)
    com.write(dev, "DSP0\\\\\\S", 8)
   	ipc.sleep(20)
    com.write(dev, "DSP1PD \\", 8)

end

function DspSPD_FLCH_on ()

	if dev == 0 then return end
    if _MCP1 () then return end

   	ipc.sleep(20)
    com.write(dev, "DSP0\\\\\\F", 8)
   	ipc.sleep(20)
    com.write(dev, "DSP1LC*\\", 8)

end

function DspSPD_FLCH_off ()

	if dev == 0 then return end
    if _MCP1 () then return end

   	ipc.sleep(20)
    com.write(dev, "DSP0\\\\\\S", 8)
   	ipc.sleep(20)
    com.write(dev, "DSP1PD \\", 8)

end

function DspLNAV_on ()

	if dev == 0 then return end
    if _MCP1 () then return end

   	ipc.sleep(20)
    com.write(dev, "DSP4LNAV", 8)

end

function DspLNAV_off ()

	if dev == 0 then return end
    if _MCP1 () then return end

   	ipc.sleep(20)
    com.write(dev, "DSP4HDG ", 8)

end


function DspVNAV_on ()

	if dev == 0 then return end
    if _MCP1 () then return end

   	ipc.sleep(20)
    com.write(dev, "DSP5  VN", 8)
   	ipc.sleep(20)
    com.write(dev, "DSP6AV  ", 8)

end

function DspVNAV_off ()

	if dev == 0 then return end
    if _MCP1 () then return end

   	ipc.sleep(20)
    com.write(dev, "DSP5  AL", 8)
   	ipc.sleep(20)
    com.write(dev, "DSP6T   ", 8)

end

function DspHDG_AP_on ()

	if dev == 0 then return end
    if _MCP1 () then return end

   	ipc.sleep(20)
    com.write(dev, "DSPC\\\\\\*", 8)

end

function DspHDG_AP_off ()

	if dev == 0 then return end
    if _MCP1 () then return end

   	ipc.sleep(20)
    com.write(dev, "DSPC\\\\\\ ", 8)

end

function DspALT_AP_on ()

	if dev == 0 then return end
    if _MCP1 () then return end

   	ipc.sleep(20)
    com.write(dev, "DSPE\\\\* ", 8)

end

function DspALT_AP_off ()

	if dev == 0 then return end
    if _MCP1 () then return end

   	ipc.sleep(20)
    com.write(dev, "DSPE\\\\  ", 8)

end

function DspVVS_AP_on ()

	if dev == 0 then return end
    if _MCP1 () then return end

   	ipc.sleep(20)
    com.write(dev, "DSP7\\\\\\*", 8)

end

function DspVVS_AP_off ()

	if dev == 0 then return end
    if _MCP1 () then return end

   	ipc.sleep(20)
    com.write(dev, "DSP7\\\\\\ ", 8)

end


function DspCRS (i)

	if dev == 0 then return end
    if _MCP1 () then return end

	ipc.sleep(20)
	com.write(dev, "dsp3/" .. DspNum(i), 8)

end