-- MAIN EVENT HANDLER
function TimerOneSec ()

	if not init then return end
	if dev == 0 then return end

    if type(VRI_TIMER) == "function" then VRI_TIMER () end

	if type(Timer) == "function" then Timer () end

end


-- interacting with GUI
function offset_reloadConfigs (offset, value)

	if EVENTS_INIT then return end

	if value == 1 then
	
        -- reloading actions.lua
		dofile(ipc.get("PATH_ACFT") .. ipc.get("acft_handle") .. "/actions.lua")

		-- main joystick config	
		local config_hid = ipc.get("PATH_SYS_CFG") .. "config-hid.lua"
		if file_exists(config_hid) then
			
			dofile(config_hid)
			ipc.set("HID_READY", 1)
					
			-- Initialize HID devices
			hidInit ()

			-- loading fallback assignments HID config
			local config_hid = ipc.get("PATH_ACFT_CFG") .. "FSX default/config-hid.lua"
			if file_exists(config_hid) then
			
				dofile(config_hid) 
			 
			end


			-- loading this aircraft assignments HID config
			local config_hid = ipc.get("PATH_ACFT_CFG") .. ipc.get("acft_handle") .. "/config-hid.lua"
			if file_exists(config_hid) then

				dofile(config_hid) 
			 
			end		
			
		else
		
			_err ("HID devices config not found, module disabled!")
			ipc.set("HID_READY", 0)
			hidinit = false
			
		end

		-- Loading aircraft module asignments
		dofile(ipc.get("PATH_ACFT_CFG") .. ipc.get("acft_handle") .. "/config-" .. ipc.get("VRItype") .. ".lua")

		-- drop reload flag
		ipc.writeUB(x_RELOAD,0)
	
		_log ('Configs reloaded!')
		
	end
	
end


function offset_executeCommand (offset, value)
	
	if EVENTS_INIT then return end

	value = string.format("%s", value)


	if value ~= '' then
		
		_logg ('Command request: [' .. value .. ']')
		
		ipc.writeSTR(x_EXEC,'',60)
		
		if type(_G[value]) == 'function' then 
		
			ipc.writeUB(x_QUEUE, 0)
			_log ('[S] GUI Command request: [' .. value .. ']')
			_G[value] () 

		else
		
			-- _logg ('GUI Special command request: [' .. value .. ']')
			
			if ipc.get("HID_ENABLED") == 1 then 

				if string.find(value, "PR:") ~= nil then
				
					-- button press
				
					buttonOnPress (string.sub(value, 4, 12), tonumber(string.sub(value, 14)) )
					ipc.writeUB(x_QUEUE, 0)
						
					return
				
				end
		
			
				if string.find(value, "RL:") ~= nil then
				
					-- button press
				
					buttonOnRelease (string.sub(value, 4, 12), tonumber(string.sub(value, 14)) )
					ipc.writeUB(x_QUEUE, 0)
				
					return
				
				end
				
			end
			
			if string.find(value, "Control:") ~= nil then
			
				FSXcontrol (string.sub(value, 10))
				ipc.writeUB(x_QUEUE, 0)
			
				return
			
			end

			if string.find(value, "FSX:") ~= nil then
			
				FSXcontrolName (string.sub(value, 6))
				ipc.writeUB(x_QUEUE, 0)
			
				return
			
			end

			if string.sub(value, 1, 2) == "M:" then
			
				FSUIPCmacro (string.sub(value, 4))
				ipc.writeUB(x_QUEUE, 0)
			
				return
			
			end

			if string.sub(value, 1, 3) == "GL:" then
			
				-- get LVar value
			
				hunterGetLVar (string.sub(value, 4))
				ipc.writeUB(x_QUEUE, 0)
			
				return
			
			end
			
			if string.sub(value, 1, 3) == "CL:" then
			
				-- check LVar value
			
				hunterCheckLVar (string.sub(value, 4))
				ipc.writeUB(x_QUEUE, 0)
			
				return
			
			end

			if string.sub(value, 1, 3) == "SL:" then
			
				-- set LVar value
			
				hunterSetLVar (string.sub(value, 4))
				ipc.writeUB(x_QUEUE, 0)
			
				return
			
			end
			
			if string.sub(value, 1, 3) == "DL:" then
			
				-- remove LVar from watching list
			
				hunterRemoveLVar (string.sub(value, 4))
				ipc.writeUB(x_QUEUE, 0)
			
				return
			
			end
						
						
			if string.sub(value, 1, 3) == "OR:" then
			
				-- offset check
			
				hunterOffsetRead (string.sub(value, 4))
				ipc.writeUB(x_QUEUE, 0)
			
				return
			
			end
			
			if string.sub(value, 1, 3) == "OW:" then
			
				-- offset watch
			
				hunterOffsetAdd (string.sub(value, 4))
				ipc.writeUB(x_QUEUE, 0)
			
				return
			
			end
			
			
			if string.sub(value, 1, 3) == "OD:" then
			
				-- offset stop watch
			
				hunterOffsetRemove (string.sub(value, 4))
				ipc.writeUB(x_QUEUE, 0)
			
				return
			
			end
			
			if string.sub(value, 1, 3) == "OS:" then
			
				-- offset set
			
				hunterOffsetSet (string.sub(value, 4))
				ipc.writeUB(x_QUEUE, 0)
			
				return
			
			end
			
			
			if string.sub(value, 1, 3) == "FC:" then
			
				-- fsx control
			
				hunterFSXcontrol (string.sub(value, 4))
				ipc.writeUB(x_QUEUE, 0)
			
				return
			
			end
			
			
			if string.sub(value, 1, 3) == "LG:" then
			
				-- offset watch
			
				hunterFSUIPClogging (string.sub(value, 4))
				ipc.writeUB(x_QUEUE, 0)
			
				return
			
			end
			
		end


		-- if command is unrecognized then clear flag ANYWAY
		ipc.writeUB(x_QUEUE, 0)

	end
	
end


-- reading HID devices
function hidInit ()

	if ipc.get("HID_READY") ~= 1 then return end

	local Device = 0  -- Multiple devices of the same name need increasing Device numbers.
	local Report = 0
	
	for jid = 1, J[0] do
	
		if J[jid][4] == 1 then
		
			-- generating full dev id
			VID = tonumber(string.sub(J[jid][1],1,4), 16)
			PID = tonumber(string.sub(J[jid][1],5,8), 16)
			UID = tonumber(string.sub(J[jid][1],9))
			
			-- _log(VID .. " / " .. PID .. " / " .. UID )

			-- jdev[jid], rd[jid], wrf[jid], wr[jid], init1[jid] = 
			--	com.openhid(VID, PID, UID, Report) 
			
			prevbuttons[jid] = 0
			SHIFT_LOC[jid] = 0

			-- init assignments tables
				JSTK[J[jid][1]]={}
			  JSTKrp[J[jid][1]]={}
			  JSTKrl[J[jid][1]]={}

			   JSTK2[J[jid][1]]={}
			 JSTK2rp[J[jid][1]]={}
			 JSTK2rl[J[jid][1]]={}
				 
			jdev[jid] = 1	 
			-- if dev found and active...
			if jdev[jid] ~= 0 then
			
				_logg ("Joystick found: " .. J[jid][2] .. ' ['.. tostring(jdev[jid]) .. ']')
				
				-- saving jid ->> devid table
				JDEVID[jid] = J[jid][1]
				
				-- reading and parsing hats string
				if J[jid][7] ~= "" then
				
					-- init hat tables
					prevval[jid] = {}
					JHATS[jid] = {}
					
					JHATS[jid] = split(J[jid][7], ",")

					-- saving number of hats for this device
					J[jid][7] = tablelength(JHATS[jid])
					
					-- init tables for hats
					for ii = 1, J[jid][7] do
					
						prevval[jid][ii] = -1
						newval[jid] = {}
					
					end
					
					-- _log("Hat:  " .. JHATS[jid][1])
					-- _log("Hats: " .. tostring(J[jid][7]))
				
				else
				
					J[jid][7] = 0
					
				end
				
				jcount = jcount + 1
			
				hidinit = true
				
			end
			
		end -- if enabled

	end

end


function hidHandle (jid, prev, curr)
	
	local b = 0

	-- bprev = Hex2Bin(string.format("%08x", prev))
	bcurr = Hex2Bin(string.format("%08x", curr))
	bxor = Hex2Bin(string.format("%08x", logic.Xor(prev, curr)))
	
	-- _log(JDEVID[jid] .. ' : ' .. bcurr)
	
	-- scan for what is changed
	for i = 1, 32 do

		b = 33 - i
	
		if string.sub(bxor, i, i) == '1' then
		
			if string.sub(bcurr, i, i) == '1' then
			-- pressed
			
				_log(JDEVID[jid] .. ' press : ' .. bcurr .. ' / ' .. tostring(b))
				
				buttonOnPress (JDEVID[jid], b)
			
			else
			-- released
			
				_log(JDEVID[jid] .. ' release : ' .. bcurr .. ' / ' .. tostring(b))
				
				buttonOnRelease (JDEVID[jid], b)
				
			end
		
		end
	
	end

end


function hidHandleHat (jid, hat, val, prev)

	local b = 280 + (20 * hat) + val + 1
	local bp = 280 + (20 * hat) + prev + 1
	
	-- _log(JDEVID[jid] .. ' : ' .. b)

	if val > -1 then
	-- pressed
		
		buttonOnPress (JDEVID[jid], b)
		buttonOnRelease (JDEVID[jid], bp)
			
	
	else
	-- released
	
		buttonOnRelease (JDEVID[jid], bp)
		
	end

end


sec = 0
sec10 = 0
function hidPoll ()

	sec = sec + 1 
	if sec > 13 then
		
		sec = 0
		TimerOneSec ()
		
	end

	sec10 = sec10 + 1
	if sec10 > 2 then
		
		sec10 = 0
		if ipc.get("HID_ENABLED") == 1 then buttonRepeatter () end
		if HUNTER then hunterGetLVarMonitor () end
		
	end
	
	-- EXITING HERE --
	if true then return end
	
	if not hidinit then return end

	for i, devid in pairs(JDEVID) do
	
		CurrentData, n = com.readlast(jdev[i], rd[i])
		-- _log(tostring(jdev[i]) .. ' -- ' .. tostring(d))
--		CurrentData[i], n[i] = com.read(jdev[i], rd[i])
		
		if n ~= 0 then
		
			-- Now handle the buttons : up to 256 of them!
			buttons[i] = com.GetHidButtons(jdev[i], CurrentData)
				
			if buttons[i] ~= prevbuttons[i] then
				
				hidHandle(i, prevbuttons[i], buttons[i])
				
				prevbuttons[i] = buttons[i]
				
			end

			-- if joystick has a hat switches..
			if J[i][7] > 0 then 
			
				for ii = 1, J[i][7] do
				
					newval[i][ii] = com.GetHidValue(jdev[i], tonumber(JHATS[i][ii]), CurrentData)
					if newval[i][ii] ~= prevval[i][ii] then
					
						hidHandleHat(i, ii, newval[i][ii], prevval[i][ii])
					
						prevval[i][ii] = newval[i][ii]
					
					end
					
				end
			
			end
			
			
		end

	end
	
end


-- SYNC BACKS
function SyncBackSPD (offset, value)

	if ipc.elapsedtime() - ipc.get("FIP") > 500 then
	
		if value ~= sync_spd then DspSPD(value) end
		
	end
	
end

function SyncBackHDG (offset, value)

	if ipc.elapsedtime() - ipc.get("FIP") > 500 then

		value = value/65536*360
		if value ~= sync_hdg then DspHDG(value) end
	
	end
	
end

function SyncBackALT (offset, value)

	if ipc.elapsedtime() - ipc.get("FIP") > 500 then

		value = value/65536*3.28084/100
		
		if value ~= sync_alt then DspALT(value) end
		
	end

end

