if ipc.get("acft_handle") == nil then

	ipc.log("LINDA: Check the loader enabled and started!")
	return

end

-- communication offsets
x_RELOAD = 0x7320   -- 0x66C0 -- 1/0 reload configs
x_EXEC = 0x7321     -- 0x66C1 -- action string
x_QUEUE = 0x735F     -- 0x66FF -- queue flag - 1:busy, 0:ready to receive


-- var hunter
require(ipc.get("PATH_SYS") .. "hunter")

-- Loading common functions and actions
require(ipc.get("PATH_SYS") .. "common")

-- loading optional libraries
if ipc.get("LIBREQ") ~= "" then
	
	libs = split(ipc.get("LIBREQ"), "#")
	
	for i,req in pairs(libs) do 
	
		_logg("INIT loading optional library: " .. req)

		require(req)
	
	end
	
end

-- Initialize system variables
CommonInitVars ()


-- main joystick config	
if ipc.get("HID_ENABLED") == 1 then 

	require(ipc.get("PATH_SYS_CFG") .. "config-hid")
	 
end


-- Loading aircraft module
require(ipc.get("PATH_ACFT") .. ipc.get("acft_handle") .. "/actions")

-- Loading aircraft module asignments
if ipc.get("VRI_ENABLED") == 1 then 

	config_mcp = ipc.get("PATH_ACFT_CFG") .. ipc.get("acft_handle") .. "/config-" .. ipc.get("VRItype")
	if file_exists(config_mcp .. ".lua") then

		require(config_mcp)
		
	else

		_err("MCP configs not found for current aircraft!")
		ipc.display("MCP configs not found for current aircraft. Run GUI!" , 10)

	end

end
	
-- Loading main engine
if ipc.get("HID_ENABLED") == 1 then require(ipc.get("PATH_SYS") .. "handlers-hid") end
if ipc.get("VRI_ENABLED") == 1 then

    require(ipc.get("PATH_SYS") .. "handlers-" .. ipc.get("VRItype"))

end
require(ipc.get("PATH_SYS") .. "events")


-- Initialize finally
Init ()


--if HID config found and ready
if ipc.get("HID_ENABLED") == 1 then 

	-- Initialize HID devices
	hidInit ()

	-- loading fallback assignments HID config
	local config_hid = ipc.get("PATH_ACFT_CFG") .. "FSX default/config-hid"
	if file_exists(config_hid .. ".lua") then

		_log("Loading a fallback joysticks config...")
		require(config_hid) 
	 
	end


	-- loading this aircraft assignments HID config
	local config_hid = ipc.get("PATH_ACFT_CFG") .. ipc.get("acft_handle") .. "/config-hid"
	if file_exists(config_hid .. ".lua") then

		_log("Loading " .. ipc.get("acft_handle") .. " joysticks config...")
		require(config_hid) 
	 
	end

end
	

_log("Module: " .. ipc.get("acft_handle") .. " Started...")

if ipc.get("VRI_ENABLED") ~= 1 then ipc.display('LINDA is started! Ready to go!', 5) end
_log("Ready to go, Captain!")
_log("")
	
-- Event to catch VRI commands from COM-port qqqqqqq
if ipc.get("VRI_ENABLED") == 1 and dev > 0 then 

	-- main event
	-- event.com (dev, 8, 8, -1, 'MCPcontrols')
	event.vriread(dev, "MCPcontrols")
	
	
	-- sync back
	event.offset(0x07E2, "UW", "SyncBackSPD")
	event.offset(0x07CC, "UW", "SyncBackHDG")
	event.offset(0x07D4, "UD", "SyncBackALT")

end

EVENTS_INIT = true

event.offset(x_RELOAD, "UB", 1, 'offset_reloadConfigs')
event.offset(x_EXEC, "STR", 60, 'offset_executeCommand')
event.timer(1000/30, "hidPoll") -- main timer event

EVENTS_INIT = false

if _MCP2 () then Default_COM_1_init () end

-- drop commands queue flag
ipc.writeUB(x_QUEUE, 0)